@@ -6,7 +6,7 @@
 /* Descr.: boolean and Meta-constraint predicate management - C part       */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,6 +22,8 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 
+#define OBJ_INIT Fd_Bool_Initializer
+
 #include "engine_pl.h"
 #include "bips_pl.h"
 
@@ -98,8 +100,6 @@ static Bool (*func_tbl[NB_OF_OP+2])();
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Bool_Meta_Initializer)
-
 static
 WamWord  *Simplify              (int sign,WamWord e_word);
 
@@ -189,10 +189,10 @@ Bool      Fd_Lte_2              (WamWord le_word,WamWord re_word);
 
 
 /*-------------------------------------------------------------------------*/
-/* BOOL_META_INITIALIZER                                                   */
+/* FD_BOOL_INITIALIZER                                                     */
 /*                                                                         */
 /*-------------------------------------------------------------------------*/
-static void Bool_Meta_Initializer(void)
+static void Fd_Bool_Initializer(void)
 
 {
  bool_tbl[NOT   ]=Functor_Arity(Create_Atom("#\\"), 1);
@@ -391,7 +391,7 @@ static WamWord *Simplify(int sign,WamWord e_word)
          return exp;
 
      case ATM:
-         word=Put_Structure(atom_char['/'],2);
+         word=Put_Structure(ATOM_CHAR('/'),2);
          Unify_Value(e_word);
          Unify_Integer(0);
          goto type_error;
@@ -418,7 +418,7 @@ type_error:
 
      if (op==NB_OF_OP)
         {
-         word=Put_Structure(atom_char['/'],2);
+         word=Put_Structure(ATOM_CHAR('/'),2);
          Unify_Atom(Functor(adr));
          Unify_Integer(Arity(adr));
          goto type_error;
@@ -774,11 +774,11 @@ static void Display_Stack(WamWord *exp)
           break;          
           
       case ZERO:
-          printf("0");
+          DBGPRINTF("0");
           break;
 
       case ONE:
-          printf("1");
+          DBGPRINTF("1");
           break;
 
       default:
