@@ -6,7 +6,7 @@
 /* Descr.: mathematical support                                            */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -25,6 +25,8 @@
 #include <stdarg.h>
 #include <string.h>
 
+#define OBJ_INIT Math_Supp_Initializer
+
 #define MATH_SUPP_FILE
 
 #include "engine_pl.h"
@@ -143,8 +145,6 @@ static Bool     sort;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Math_Supp_Initializer)
-
 static 
 Bool      Load_Left_Right_Rec   (Bool optim_eq,
                                  WamWord le_word,WamWord re_word,
@@ -188,7 +188,7 @@ void      Write_1               (WamWord term_word);
 #endif
 
 #define New_Tagged_Fd_Variable  (Tag_Value(REF,Fd_New_Variable()))
-#define Add_Cst_To_Poly(p,s,w)  (p->c_word+=(s<0) ? -(w) : (w))
+#define Add_Cst_To_Poly(p,s,w)  (p->c_word+=(s<0) ? -((long) (w)) : (w))
 
 #define New_Poly(p)             ((p).c_word=(p).nb_monom=0)
 #define Tag0_Mul(x,y)           ((x)*((y)>>TAG_SIZE))
@@ -204,13 +204,13 @@ void      Write_1               (WamWord term_word);
 static void Math_Supp_Initializer(void)
 
 {
- arith_tbl[PLUS_1]    =Functor_Arity(atom_char['+'],         1);
- arith_tbl[PLUS_2]    =Functor_Arity(atom_char['+'],         2);
- arith_tbl[MINUS_1]   =Functor_Arity(atom_char['-'],         1);
- arith_tbl[MINUS_2]   =Functor_Arity(atom_char['-'],         2);
- arith_tbl[TIMES_2]   =Functor_Arity(atom_char['*'],         2);
+ arith_tbl[PLUS_1]    =Functor_Arity(ATOM_CHAR('+'),         1);
+ arith_tbl[PLUS_2]    =Functor_Arity(ATOM_CHAR('+'),         2);
+ arith_tbl[MINUS_1]   =Functor_Arity(ATOM_CHAR('-'),         1);
+ arith_tbl[MINUS_2]   =Functor_Arity(ATOM_CHAR('-'),         2);
+ arith_tbl[TIMES_2]   =Functor_Arity(ATOM_CHAR('*'),         2);
  arith_tbl[POWER_2]   =Functor_Arity(Create_Atom("**"),      2);
- arith_tbl[DIV_2]     =Functor_Arity(atom_char['/'],         2);
+ arith_tbl[DIV_2]     =Functor_Arity(ATOM_CHAR('/'),         2);
  arith_tbl[MIN_2]     =Functor_Arity(Create_Atom("min"),     2);
  arith_tbl[MAX_2]     =Functor_Arity(Create_Atom("max"),     2);
  arith_tbl[DIST_2]    =Functor_Arity(Create_Atom("dist"),    2);
@@ -327,7 +327,8 @@ static Bool Load_Left_Right_Rec(Bool optim_eq,
 
  if (sort || p.nb_monom>MAX_MONOMS/2)
     {
-     qsort(p.m,p.nb_monom,sizeof(Monom),(int (*)())Compar_Monom);
+     qsort(p.m,p.nb_monom,sizeof(Monom),
+           (int (*)(const void *,const void *))Compar_Monom);
 
      for(i=0;i<p.nb_monom;i++)                 /* find left monomial terms */
          if (p.m[i].a_word<=0)
@@ -711,7 +712,7 @@ static Bool Normalize(WamWord e_word,int sign,Poly *p)
          return TRUE;
 
      case ATM:
-         word=Put_Structure(atom_char['/'],2);
+         word=Put_Structure(ATOM_CHAR('/'),2);
          Unify_Value(e_word);
          Unify_Integer(0);
          goto type_error;
@@ -1036,7 +1037,7 @@ end_div:
          return TRUE;
 
      default:
-         word=Put_Structure(atom_char['/'],2);
+         word=Put_Structure(ATOM_CHAR('/'),2);
          Unify_Atom(Functor(adr));
          Unify_Integer(Arity(adr));
          goto type_error;
