@@ -6,7 +6,7 @@
 /* Descr.: FD Operation support                                            */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -110,7 +110,7 @@ unsigned Nth_Root_Dn(unsigned y,unsigned n)
 
  bit=Most_Significant_Bit(y);
 
- if (bit+1<n)
+ if ((unsigned) (bit+1)<n)
      return 1;
 
  nb=bit/n;
@@ -363,59 +363,6 @@ static unsigned Find_Expon_General(unsigned x,unsigned y,unsigned *pxn)
 
 
 
-/*-------------------------------------------------------------------------*/
-/* ZERO_POWER                                                              */
-/*                                                                         */
-/*-------------------------------------------------------------------------*/
-unsigned Zero_Power(unsigned min_n,unsigned max_n,unsigned *max_y)
-
-{
- if (max_n==0)
-    {
-     *max_y=1;
-     return 1;
-    }
-
-
- if (min_n>0)
-    {
-     *max_y=0;
-     return 0;
-    }
-
- *max_y=1;
- return 0;
-}
-
-
-
-
-/*-------------------------------------------------------------------------*/
-/* ZERO_FIND_EXPON                                                         */
-/*                                                                         */
-/*-------------------------------------------------------------------------*/
-unsigned Zero_Find_Expon(unsigned min_y,unsigned max_y,unsigned *max_n)
-
-{
- if (min_y==1)                           /* here we know that Y is in 0..1 */
-    {
-     *max_n=0;
-     return 0;
-    }
-
- if (max_y==0)
-    {
-     *max_n=INTERVAL_MAX_INTEGER;
-     return 1;
-    }
-
- *max_n=INTERVAL_MAX_INTEGER;
- return 0;
-}
-
-
-
-
 /*-------------------------------------------------------------------------*/
 /* FULL_COEFF_POWER_VAR                                                    */
 /*                                                                         */
@@ -429,7 +376,7 @@ void Full_Coeff_Power_Var(Range *y,int a,Range *n)
  an=Power(a,n->min);
 
  Vector_Allocate(y->vec);
- if (an>vec_max_integer)
+ if (an > (unsigned) vec_max_integer)
     {
      y->extra_cstr=TRUE;
      Set_To_Empty(y);
@@ -445,7 +392,7 @@ void Full_Coeff_Power_Var(Range *y,int a,Range *n)
      an0=an;
      for(i=n->min;i<=n->max;i++)
         {
-         if (an0>vec_max_integer)
+         if (an0 > (unsigned) vec_max_integer)
              goto end_loop;
 
          an=an0;
@@ -460,7 +407,7 @@ void Full_Coeff_Power_Var(Range *y,int a,Range *n)
      VECTOR_BEGIN_ENUM(n->vec,vec_elem)
 
          an=Power(a,vec_elem);
-         if (an>vec_max_integer)
+         if (an > (unsigned) vec_max_integer)
              goto end_loop;
 
          Vector_Set_Value(y->vec,an);
@@ -545,7 +492,7 @@ void Full_Var_Power_Coeff(Range *y,Range *x,int a)
  xa=Power(x->min,a);
 
  Vector_Allocate(y->vec);
- if (xa>vec_max_integer)
+ if (xa > (unsigned) vec_max_integer)
     {
      y->extra_cstr=TRUE;
      Set_To_Empty(y);
@@ -561,7 +508,7 @@ void Full_Var_Power_Coeff(Range *y,Range *x,int a)
      for(i=x->min;i<=x->max;i++)
         {
          xa=Power(i,a);
-         if (xa>vec_max_integer)
+         if (xa > (unsigned) vec_max_integer)
              goto end_loop;
 
          Vector_Set_Value(y->vec,xa);
@@ -574,7 +521,7 @@ void Full_Var_Power_Coeff(Range *y,Range *x,int a)
      VECTOR_BEGIN_ENUM(x->vec,vec_elem)
 
          xa=Power(vec_elem,a);
-         if (xa>vec_max_integer)
+         if (xa > (unsigned) vec_max_integer)
              goto end_loop;
 
          Vector_Set_Value(y->vec,xa);
@@ -658,7 +605,7 @@ void Full_Var_Power_2(Range *y,Range *x)
  x2=x->min*x->min;
 
  Vector_Allocate(y->vec);
- if (x2>vec_max_integer)
+ if (x2 > (unsigned) vec_max_integer)
     {
      y->extra_cstr=TRUE;
      Set_To_Empty(y);
@@ -674,7 +621,7 @@ void Full_Var_Power_2(Range *y,Range *x)
      for(i=x->min;i<=x->max;i++)
         {
          x2=i*i;
-         if (x2>vec_max_integer)
+         if (x2 > (unsigned) vec_max_integer)
              goto end_loop;
 
          Vector_Set_Value(y->vec,x2);
@@ -687,7 +634,7 @@ void Full_Var_Power_2(Range *y,Range *x)
      VECTOR_BEGIN_ENUM(x->vec,vec_elem)
 
          x2=vec_elem*vec_elem;
-         if (x2>vec_max_integer)
+         if (x2 > (unsigned) vec_max_integer)
              goto end_loop;
 
          Vector_Set_Value(y->vec,x2);
