@@ -6,7 +6,7 @@
 /* Descr.: FD Operation support - header file                              */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -52,11 +52,6 @@ unsigned  Find_Expon_Dn         (unsigned x,unsigned y);
 unsigned  Find_Expon_Up         (unsigned x,unsigned y);
 unsigned  Find_Expon_Exact      (unsigned x,unsigned y);
 
-unsigned  Zero_Power            (unsigned min_n,unsigned max_n,
-                                 unsigned *max_y);
-unsigned  Zero_Find_Expon       (unsigned min_y,unsigned max_y,
-                                 unsigned *max_n);
-
 void      Full_Coeff_Power_Var  (Range *y,int a,Range *n);
 void      Full_Find_Expon       (Range *n,int a,Range *y);
 
