@@ -6,7 +6,7 @@
 /* Descr.: all solution collector management - C part                      */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -21,13 +21,18 @@
 /* with this program; if not, write to the Free Software Foundation, Inc.  */
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
-#include <unistd.h>
 #include <sys/types.h>
-#include <sys/wait.h>
+
+#define OBJ_INIT All_Solut_Initializer
 
 #include "engine_pl.h"
 #include "bips_pl.h"
 
+#ifndef M_ix86_win32
+#include <unistd.h>
+#include <sys/wait.h>
+#endif
+
 
 
 
@@ -81,8 +86,6 @@ static long    *next_key_var_ptr;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(All_Solut_Initializer)
-
 static
 void      Bound_Var            (WamWord *adr);
 static
@@ -118,7 +121,7 @@ Prolog_Prototype(GROUP_SOLUTIONS_ALT,0)
 static void All_Solut_Initializer(void)
 
 {
- exist_2=Functor_Arity(atom_char['^'],2);
+ exist_2=Functor_Arity(ATOM_CHAR('^'),2);
 }
 
 
@@ -167,7 +170,7 @@ Bool Free_Variables_4(WamWord templ_word,WamWord gen_word,WamWord gen1_word,
 
  if (nb_free_var<=MAX_ARITY)
     {
-     *save_H=Functor_Arity(atom_char['.'],nb_free_var);
+     *save_H=Functor_Arity(ATOM_CHAR('.'),nb_free_var);
      gl_key_word=Tag_Value(STC,save_H);
     }
   else
