@@ -6,7 +6,7 @@
 /* Descr.: arithmetic (inline) management - C part                         */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -25,9 +25,15 @@
 #include <string.h>
 #include <math.h>
 
+#define OBJ_INIT Arith_Initializer
+
 #include "engine_pl.h"
 #include "bips_pl.h"
 
+#ifdef M_ix86_win32
+#define rint(x)  (floor((x)+(double) 0.5))
+#endif
+
 
 
 
@@ -66,8 +72,6 @@ static char *arith_tbl;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Arith_Initializer)
-
 static
 int       Error_Zero_Divisor    (void);
 
@@ -494,7 +498,7 @@ static WamWord Load_Math_Expression(WamWord exp)
          return word;
 
      case ATM:
-         word=Put_Structure(atom_char['/'],2);
+         word=Put_Structure(ATOM_CHAR('/'),2);
          Unify_Value(exp);
          Unify_Integer(0);
          break;                                         /* then type_error */
@@ -504,8 +508,8 @@ static WamWord Load_Math_Expression(WamWord exp)
          Deref(Cdr(lst_adr),word,tag,adr);
          if (word!=NIL_WORD)
             {
-             word=Put_Structure(atom_char['/'],2);
-             Unify_Atom(atom_char['.']);
+             word=Put_Structure(ATOM_CHAR('/'),2);
+             Unify_Atom(ATOM_CHAR('.'));
              Unify_Integer(2);
              Pl_Err_Type(type_evaluable,word);
             }
@@ -517,7 +521,7 @@ static WamWord Load_Math_Expression(WamWord exp)
          arith=(ArithInf *) Hash_Find(arith_tbl,Functor_And_Arity(adr));
          if (arith==NULL)
             {
-             word=Put_Structure(atom_char['/'],2);
+             word=Put_Structure(ATOM_CHAR('/'),2);
              Unify_Atom(Functor(adr));
              Unify_Integer(Arity(adr));
              Pl_Err_Type(type_evaluable,word);
