@@ -6,7 +6,7 @@
 /* Descr.: atom manipulation management - C part                           */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -52,6 +52,9 @@ Bool      Compute_Next_BLA      (int mask,AtomInf *patom,AtomInf *psub_atom,
                                  int b,int l,int a,int *b1,int *l1,int *a1);
 
 
+static
+int       Create_Malloc_Atom    (char *str);
+
 static 
 Bool      String_To_Number      (char *str,WamWord number_word);
 
@@ -144,7 +147,7 @@ Bool Atom_Concat_3(WamWord atom1_word,WamWord atom2_word,WamWord atom3_word)
          MALLOC_STR(l)
          strcpy(str,patom1->name);
          strcpy(str+patom1->prop.length,patom2->name);
-         return Get_Atom(Create_Atom(str),atom3_word);
+         return Get_Atom(Create_Malloc_Atom(str),atom3_word);
         }
 
      patom3=atom_tbl+UnTag_ATM(atom3_word);
@@ -153,7 +156,7 @@ Bool Atom_Concat_3(WamWord atom1_word,WamWord atom2_word,WamWord atom3_word)
      strcpy(str,patom3->name+patom1->prop.length);
 
      return strncmp(patom1->name,patom3->name,patom1->prop.length)==0 &&
-            Get_Atom(Create_Atom(str),atom2_word);
+            Get_Atom(Create_Malloc_Atom(str),atom2_word);
     }
 
  if (tag2==ATM)                                          /* here tag1==REF */
@@ -166,7 +169,7 @@ Bool Atom_Concat_3(WamWord atom1_word,WamWord atom2_word,WamWord atom3_word)
      str[l]='\0';
 
      return strncmp(patom2->name,patom3->name+l,patom2->prop.length)==0 &&
-            Get_Atom(Create_Atom(str),atom1_word);
+            Get_Atom(Create_Malloc_Atom(str),atom1_word);
     }
 
                         /* A1 and A2 are variables: non deterministic case */
@@ -228,12 +231,12 @@ Bool Atom_Concat_Alt_0(void)
  MALLOC_STR(l)
  strncpy(str,name,l+1);
  str[l]='\0';
- Get_Atom(Create_Atom(str),atom1_word);
+ Get_Atom(Create_Malloc_Atom(str),atom1_word);
 
  l=patom3->prop.length-l;
  MALLOC_STR(l)
  strcpy(str,p);
- Get_Atom(Create_Atom(str),atom2_word);
+ Get_Atom(Create_Malloc_Atom(str),atom2_word);
 
  return TRUE;
 }
@@ -383,7 +386,7 @@ Bool Sub_Atom_5(WamWord atom_word,WamWord before_word,WamWord length_word,
      MALLOC_STR(l)
      strncpy(str,patom->name+b,l);
      str[l]='\0';
-     Get_Atom(Create_Atom(str),sub_atom_word);
+     Get_Atom(Create_Malloc_Atom(str),sub_atom_word);
      Get_Integer(l,length_word);
     }
 
@@ -443,7 +446,7 @@ Bool Sub_Atom_Alt_0(void)
      MALLOC_STR(l)
      strncpy(str,patom->name+b,l);
      str[l]='\0';
-     Get_Atom(Create_Atom(str),sub_atom_word);
+     Get_Atom(Create_Malloc_Atom(str),sub_atom_word);
      Get_Integer(l,length_word);
     }
 
@@ -515,6 +518,25 @@ static Bool Compute_Next_BLA(int mask,AtomInf *patom,AtomInf *psub_atom,
 
 
 
+/*-------------------------------------------------------------------------*/
+/* CREATE_MALLOC_ATOM                                                      */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static int Create_Malloc_Atom(char *str)
+
+{
+ int atom;
+ int nb=nb_atom;
+
+ atom=Create_Atom(str);
+ if (nb==nb_atom)
+     Free(str);
+ return atom;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*/
 /* ATOM_CHARS_2                                                            */
 /*                                                                         */
