@@ -6,7 +6,7 @@
 /* Descr.: byte-code support                                               */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#define OBJ_INIT Byte_Code_Initializer
 
 #define BC_SUPP_FILE
 
@@ -190,8 +191,6 @@ static Bool     debug_call;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Byte_Code_Initializer)
-
 static
 int       Find_Inst_Code_Op     (int inst);
 static
@@ -334,7 +333,8 @@ static void Byte_Code_Initializer(void)
 
  nb_op=p-op_tbl;
 
- qsort(op_tbl,nb_op,sizeof(op_tbl[0]),(int(*)())Compar_Inst_Code_Op);
+ qsort(op_tbl,nb_op,sizeof(op_tbl[0]),
+       (int(*)(const void *,const void *))Compar_Inst_Code_Op);
 
 
  bc_nb_block=1;
@@ -369,7 +369,7 @@ static int Find_Inst_Code_Op(int inst)
 
  BC_Atm(w)=inst;
  p=(BCWord *) bsearch(&w,op_tbl,nb_op,sizeof(op_tbl[0]),
-                      (int(*)())Compar_Inst_Code_Op);
+                      (int(*)(const void *,const void *))Compar_Inst_Code_Op);
  if (p==NULL)
      Fatal_Error(ERR_UNKNOWN_INSTRUCTION,atom_tbl[inst].name);
 
@@ -514,7 +514,7 @@ void BC_Emit_Inst_1(WamWord inst_word)
  op=Find_Inst_Code_Op(func);
 
  size_bc=bc_sp-bc;
- if (size_bc+3>=bc_nb_block)
+ if (size_bc+3>=bc_nb_block*BC_BLOCK_SIZE)
     {
      bc_nb_block++;
      bc=(BCWord *) Realloc((char *) bc,
@@ -648,10 +648,14 @@ void BC_Emit_Inst_1(WamWord inst_word)
  *bc_sp++=w;
  if (nb_word>=2)
     {
-     *bc_sp++=(BCWord) w1;
+     bc_sp->word=w1;
+     bc_sp++;
 
      if (nb_word>=3)
-         *bc_sp++=(BCWord) w2;
+        {
+         bc_sp->word=w2;
+         bc_sp++;
+        }
     }
 
 
@@ -689,7 +693,7 @@ static int BC_Arg_X_Or_Y(WamWord arg_word,int *op)
  Deref(arg_word,word,tag,adr)
  adr=UnTag_STC(word);
 
- if (Functor(adr)!=atom_char['x'])
+ if (Functor(adr)!=ATOM_CHAR('x'))
      (*op)++;                              /* +1 for op when Y is involved */
 
  return BC_Arg_Integer(Arg(adr,0));
