@@ -6,7 +6,7 @@
 /* Descr.: C interface support                                             */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -1083,7 +1083,7 @@ WamWord *Rd_Compound_Check(WamWord start_word,int *func,int *arity)
 
      case LST:
          adr=UnTag_LST(word);
-         *func=atom_char['.'];
+         *func=ATOM_CHAR('.');
          *arity=2;
          return &Car(adr);
 
@@ -1116,7 +1116,7 @@ WamWord *Rd_Compound(WamWord start_word,int *func,int *arity)
     {
      case LST:
          adr=UnTag_LST(word);
-         *func=atom_char['.'];
+         *func=ATOM_CHAR('.');
          *arity=2;
          return &Car(adr);
 
@@ -1157,7 +1157,7 @@ WamWord *Rd_Callable_Check(WamWord start_word,int *func,int *arity)
 
      case LST:
          adr=UnTag_LST(word);
-         *func=atom_char['.'];
+         *func=ATOM_CHAR('.');
          *arity=2;
          return &Car(adr);
 
@@ -1195,7 +1195,7 @@ WamWord *Rd_Callable(WamWord start_word,int *func,int *arity)
 
      case LST:
          adr=UnTag_LST(word);
-         *func=atom_char['.'];
+         *func=ATOM_CHAR('.');
          *arity=2;
          return &Car(adr);
 
@@ -1555,7 +1555,7 @@ Bool Un_Number_Check(double value,WamWord start_word)
 
  CHECK_FOR_UN_NUMBER
 
- n=value;
+ n=(int) value;
 
  return (n==value) ? Get_Integer(n,word) : Get_Float(value,word);
 }
@@ -1572,7 +1572,7 @@ Bool Un_Number(double value,WamWord start_word)
 {
  int n;
 
- n=value;
+ n=(int) value;
 
  return (n==value) ? Get_Integer(n,start_word) : Get_Float(value,start_word);
 }
@@ -1645,7 +1645,7 @@ Bool Un_Char_Check(int value,WamWord start_word)
 {
  CHECK_FOR_UN_CHAR
 
- return Get_Atom(atom_char[value],word);
+ return Get_Atom(ATOM_CHAR(value),word);
 }
 
 
@@ -1658,7 +1658,7 @@ Bool Un_Char_Check(int value,WamWord start_word)
 Bool Un_Char(int value,WamWord start_word)
 
 {
- return Get_Atom(atom_char[value],start_word);
+ return Get_Atom(ATOM_CHAR(value),start_word);
 }
 
 
@@ -1673,7 +1673,7 @@ Bool Un_In_Char_Check(int value,WamWord start_word)
 {
  CHECK_FOR_UN_IN_CHAR
 
- return Get_Atom((value==-1) ? atom_end_of_file:atom_char[value],word);
+ return Get_Atom((value==-1) ? atom_end_of_file:ATOM_CHAR(value),word);
 }
 
 
@@ -1686,7 +1686,7 @@ Bool Un_In_Char_Check(int value,WamWord start_word)
 Bool Un_In_Char(int value,WamWord start_word)
 
 {
- return Get_Atom((value==-1) ? atom_end_of_file:atom_char[value],start_word);
+ return Get_Atom((value==-1) ? atom_end_of_file:ATOM_CHAR(value),start_word);
 }
 
 
@@ -1856,7 +1856,7 @@ Bool Un_Chars(char *str,WamWord start_word)
 {
  for(;*str;str++)
     {
-     if (!Get_List(start_word) || !Unify_Atom(atom_char[(unsigned) (*str)]))
+     if (!Get_List(start_word) || !Unify_Atom(ATOM_CHAR(*str)))
          return FALSE;
 
      start_word=Unify_Variable();
@@ -2002,7 +2002,7 @@ Bool Un_Compound_Check(int func,int arity,WamWord *arg,WamWord start_word)
  if (arity==0)
      return Un_Atom_Check(func,start_word);
 
- if (arity==2 && func==atom_char['.'])
+ if (arity==2 && func==ATOM_CHAR('.'))
      return Un_List_Check(arg,start_word);
 
  if (!Get_Structure(func,arity,start_word))
@@ -2034,7 +2034,7 @@ Bool Un_Compound(int func,int arity,WamWord *arg,WamWord start_word)
  if (arity==0)
      return Un_Atom_Check(func,start_word);
 
- if (arity==2 && func==atom_char['.'])
+ if (arity==2 && func==ATOM_CHAR('.'))
      return Un_List(arg,start_word);
 
  if (!Get_Structure(func,arity,start_word))
@@ -2062,7 +2062,7 @@ Bool Un_Callable_Check(int func,int arity,WamWord *arg,WamWord start_word)
  if (arity==0)
      return Un_Atom_Check(func,start_word);
 
- if (arity==2 && func==atom_char['.'])
+ if (arity==2 && func==ATOM_CHAR('.'))
      return Un_List_Check(arg,start_word);
 
  if (!Get_Structure(func,arity,start_word))
@@ -2142,7 +2142,7 @@ WamWord Mk_Number(double value)
 {
  int n;
 
- n=value;
+ n=(int) value;
 
  if (n==value)
      return Put_Integer(n);
@@ -2186,7 +2186,7 @@ WamWord Mk_Boolean(int value)
 WamWord Mk_Char(int value)
 
 {
- return Put_Atom(atom_char[value]);
+ return Put_Atom(ATOM_CHAR(value));
 }
 
 
@@ -2199,7 +2199,7 @@ WamWord Mk_Char(int value)
 WamWord Mk_In_Char(int value)
 
 {
- return Put_Atom((value==-1) ? atom_end_of_file: atom_char[value]);
+ return Put_Atom((value==-1) ? atom_end_of_file: ATOM_CHAR(value));
 }
 
 
@@ -2285,7 +2285,7 @@ WamWord Mk_Chars(char *str)
  res_word=Put_List();
  for(;;)
     {
-     Unify_Atom(atom_char[(unsigned) (*str)]);
+     Unify_Atom(ATOM_CHAR(*str));
      str++;
      if (*str=='\0')
          break;
@@ -2407,7 +2407,7 @@ WamWord Mk_Compound(int func,int arity,WamWord *arg)
  if (arity==0)
      return Put_Atom(func);
 
- if (arity==2 && func==atom_char['.'])
+ if (arity==2 && func==ATOM_CHAR('.'))
      return Mk_List(arg);
 
  res_word=Put_Structure(func,arity);
