@@ -6,7 +6,7 @@
 /* Descr.: file consulting - C part                                        */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -21,17 +21,23 @@
 /* with this program; if not, write to the Free Software Foundation, Inc.  */
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
-#include <unistd.h>
 #include <sys/types.h>
-#include <sys/wait.h>
 
 #include "engine_pl.h"
 #include "bips_pl.h"
 
-#ifdef M_ix86_cygwin
+#if defined(M_ix86_cygwin) || defined(M_ix86_win32)
 #include <process.h>
 #endif
 
+#ifdef M_ix86_win32
+#include <io.h>
+#include <fcntl.h>
+#else
+#include <unistd.h>
+#include <sys/wait.h>
+#endif
+
 
 
 
@@ -63,45 +69,28 @@
 Bool Consult_2(WamWord tmp_file_word,WamWord pl_file_word)
 
 {
-#ifdef __unix__
-
- char   *tmp_file,*pl_file;
- int     pid;
- int     pipe_out[2];
- FILE   *f_out;
+ char   *tmp_file=Rd_String_Check(tmp_file_word);
+ char   *pl_file =Rd_String_Check(pl_file_word);
+ char   *singl_warn=(Flag_Value(FLAG_SINGLETON_WARNING)) ? NULL 
+                                                         : "--no-singl-warn";
  StmInf *pstm=stm_tbl+stm_top_level_output;
+ int     pid;
+ FILE   *f_out;
  int     status,c;
- char   *singl_warn;
-
- tmp_file=Rd_String_Check(tmp_file_word);
- pl_file =Rd_String_Check(pl_file_word);
-
- singl_warn=(Flag_Value(FLAG_SINGLETON_WARNING)) ? NULL : "--no-singl-warn";
-
- Os_Test_Error(pipe(pipe_out))
+ char   *arg[]={"pl2wam","-w","--compile-msg","--no-redef-error",
+                "--pl-state",tmp_file,"-o",tmp_file,pl_file,
+                singl_warn,NULL };
 
  Flush_All_Streams();
- pid=(int) fork();
+ pid=M_Spawn_Redirect(arg,0,NULL,&f_out,&f_out);
  Os_Test_Error(pid== -1)
- 
- if (pid==0)                                          /* the child process */
+ if (pid== -2)
     {
-     close(pipe_out[0]);
-     if (pipe_out[1]!=1)
-        {
-         Os_Test_Error(dup2(pipe_out[1],1)== -1)
-         Os_Test_Error(close(pipe_out[1]))
-        }
-
-     execlp("pl2wam","pl2wam","-w","--compile-msg","--no-redef-error","--pl-state",tmp_file,
-                     "-o",tmp_file,pl_file,singl_warn,NULL);
-     exit(63);                       /* only returns on error - quit child */
+error_pl2wam:                                                                        Pl_Err_System(Create_Atom("error trying to execute pl2wam "
+                               "(maybe not found)"));
+     return FALSE;
     }
 
- Os_Test_Error(close(pipe_out[1]))
- f_out=fdopen(pipe_out[0],"rt");
- Os_Test_Error(f_out==NULL)
-
  for(;;)
     {
      c=fgetc(f_out);
@@ -113,57 +102,11 @@ Bool Consult_2(WamWord tmp_file_word,WamWord pl_file_word)
 
  fclose(f_out);
 
-
- if (waitpid(pid,&status,0)<0 || !WIFEXITED(status))
-     return FALSE;
-
- status=WEXITSTATUS(status);
- if (status==63)
+ status=M_Get_Status(pid);
+ if (status<0)
      goto error_pl2wam;
 
  return status==0;
-
-#else
-
-#ifdef M_ix86_cygwin
-
- char *tmp_file,*pl_file;
- int   status;
- char *singl_warn;
-
- tmp_file=Rd_String_Check(tmp_file_word);
- pl_file =Rd_String_Check(pl_file_word);
-
- singl_warn=(Flag_Value(FLAG_SINGLETON_WARNING)) ? NULL : "--no-singl-warn";
-
- status=spawnlp(_P_WAIT,"pl2wam","pl2wam","-w","--compile-msg",
-                "--no-redef-error","--pl-state",tmp_file,"-o",tmp_file,
-                pl_file,singl_warn,NULL);
- if (status== -1)
-     goto error_pl2wam;
- return status==0;
-
-#else
-
- char *tmp_file,*pl_file;
- static
- char cmd[1024];
-
- tmp_file=Rd_String_Check(tmp_file_word);
- pl_file =Rd_String_Check(pl_file_word);
-
- sprintf(cmd,"pl2wam -w --compile-msg --pl-state %s -o %s %s %s",
-         tmp_file,tmp_file,pl_file,
-         (Flag_Value(FLAG_SINGLETON_WARNING)) ? "" : "--no-singl-warn");;
-
- return system(cmd)==0;
-#endif /* M_ix86_cygwin */
-#endif /* __unix__ */
-
-error_pl2wam:
- Pl_Err_System(Create_Atom("error trying to execute pl2wam (maybe not found)"));
- return FALSE;
-
 }
 
 #endif
