@@ -6,7 +6,7 @@
 /* Descr.: debugger - C part                                               */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -24,6 +24,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+#define OBJ_INIT Debug_Initializer
+
 #include "engine_pl.h"
 #include "bips_pl.h"
 
@@ -85,8 +87,6 @@ static StmInf  *pstm_o;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Debug_Initializer)
-
 static
 void      Scan_Command          (char *source_str);
 
@@ -415,7 +415,6 @@ static Bool Write_Data_Modify(void)
  char    *bank_name;
  int      offset;
  int      nb;
- int      i;
 
  if ((adr=Read_Bank_Adr(FALSE,1,&bank_name))!=NULL)
     {
@@ -458,8 +457,7 @@ static Bool Write_Data_Modify(void)
 
 
  if (adr==reg_copy)                              /* saved by Read_Bank_Adr */
-     for(i=0;i<NB_OF_REGS;i++)
-         Reg(i)=reg_copy[i];
+     Restore_All_Regs(reg_copy)
 
  return FALSE;
 }
@@ -740,8 +738,7 @@ static WamWord *Read_Bank_Adr(Bool only_stack,int arg_nb,char **bank_name)
      if (strncmp("reg",read_arg[arg_nb],lg)==0)
         {
          *bank_name="reg";
-         for(i=0;i<NB_OF_REGS;i++)
-             reg_copy[i]=Reg(i);
+         Save_All_Regs(reg_copy)
          return reg_copy;
         }
     }
@@ -867,6 +864,7 @@ static void Print_Wam_Word(WamWord *word_adr)
              break;
 
          default:
+             ;
         }
     }
  else
@@ -1007,6 +1005,7 @@ tag_value:
                  goto err;
 
              default:
+                 ;
             }
         }
 
