@@ -6,7 +6,7 @@
 /* Descr.: dynamic predicate support                                       */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -49,7 +49,7 @@
 #define ALL_MUST_BE_ERASED         (DynCInf *) 2 /* NB: bit 0 used for mark*/
 #define MAX_KBYTES_BEFORE_CLEAN    32
 
-#define MAX_SIZE_BEFORE_CLEAN      (MAX_KBYTES_BEFORE_CLEAN/sizeof(WamWord))
+#define MAX_SIZE_BEFORE_CLEAN      (MAX_KBYTES_BEFORE_CLEAN*1024/sizeof(WamWord))
 
 
 #define START_DYNAMIC_SWT_SIZE     32
@@ -235,7 +235,7 @@ DynCInf *Add_Dynamic_Clause(WamWord head_word,WamWord body_word,Bool asserta,
   else
      if (check_perm && !(pred->prop & MASK_PRED_DYNAMIC))
         {
-         word=Put_Structure(atom_char['/'],2);
+         word=Put_Structure(ATOM_CHAR('/'),2);
          Unify_Atom(func);
          Unify_Integer(arity);
          Pl_Err_Permission(permission_operation_modify,
@@ -344,6 +344,7 @@ static DynPInf *Alloc_Init_Dyn_Info(PredInf *pred,int arity)
  dyn->arity=arity;
  dyn->count_a= -1;
  dyn->count_z=  0;
+ dyn->first_erased_cl=NULL;
  dyn->next_dyn_with_erase=NULL;
 
  pred->dyn=(long *) dyn;
@@ -575,6 +576,24 @@ static void Clean_Erased_Clauses(void)
          clause=clause->next_erased_cl;
          Free_Clause(clause1);
         }
+     dyn->first_erased_cl=NULL;
+     dyn->next_dyn_with_erase=NULL;
+
+     if (dyn->seq_chain==NULL)                          /* no more clauses */
+        {
+         if (dyn->atm_htbl)
+             Hash_Free_Table(dyn->atm_htbl);
+
+         if (dyn->int_htbl)
+             Hash_Free_Table(dyn->int_htbl);
+
+         if (dyn->stc_htbl)
+             Hash_Free_Table(dyn->stc_htbl);
+
+         dyn->atm_htbl=dyn->int_htbl=dyn->stc_htbl=NULL;
+         dyn->count_a= -1;
+         dyn->count_z=  0;
+        }
     }
 }
 
@@ -652,7 +671,7 @@ PredInf *Update_Dynamic_Pred(int func,int arity,int what_to_do)
 
  if ((what_to_do & 1) && !(pred->prop & MASK_PRED_DYNAMIC))
     {
-     word=Put_Structure(atom_char['/'],2);
+     word=Put_Structure(ATOM_CHAR('/'),2);
      Unify_Atom(func);
      Unify_Integer(arity);
      Pl_Err_Permission(permission_operation_modify,
