@@ -6,7 +6,7 @@
 /* Descr.: Prolog errors support                                           */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,6 +22,9 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 #include <errno.h>
+#include <string.h>
+
+#define OBJ_INIT Error_Supp_Initializer
 
 #define ERROR_SUPP_FILE
 
@@ -62,8 +65,6 @@ static char    *last_err_msg;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Error_Supp_Initializer)
-
 static 
 void      Update_Cur_From_C_Bip (void);
 static
@@ -332,7 +333,7 @@ void Context_Error_1(WamWord err_word)
 {
  if (cur_bip_arity>=0)
     {
-     Get_Structure(atom_char['/'],2,err_word);
+     Get_Structure(ATOM_CHAR('/'),2,err_word);
      Unify_Atom(cur_bip_func);
      Unify_Integer(cur_bip_arity);
     }
@@ -443,7 +444,7 @@ void Unknown_Pred_Error(int func,int arity)
 
  if (Flag_Value(FLAG_UNKNOWN)==FLAG_VALUE_ERROR)
     {
-     term=Put_Structure(atom_char['/'],2);
+     term=Put_Structure(ATOM_CHAR('/'),2);
      Unify_Atom(func);
      Unify_Integer(arity);
      Pl_Err_Existence(existence_procedure,term);
