@@ -6,7 +6,7 @@
 /* Descr.: file name management - C part                                   */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -21,13 +21,18 @@
 /* with this program; if not, write to the Free Software Foundation, Inc.  */
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
-#include <unistd.h>
-#include <sys/param.h>
 #include <string.h>
 
 #include "engine_pl.h"
 #include "bips_pl.h"
 
+#ifdef M_ix86_win32
+#include <io.h>
+#else
+#include <unistd.h>
+#include <sys/param.h>
+#endif
+
 
 
 
@@ -91,10 +96,13 @@ Bool Decompose_File_Name_4(WamWord path_word,WamWord dir_word,
  Check_For_Un_Atom(prefix_word);
  Check_For_Un_Atom(suffix_word);
 
- p=strrchr(path,'/');
-#ifdef __msdos__
- if (p==NULL)
-     p=strrchr(path,'\\');
+ p=strrchr(path,DIR_SEP_C);
+#ifdef M_ix86_win32
+{
+ char *q=strrchr(path,'/');
+ if (p==NULL || p>q)
+     p=q;
+}
 #endif
 
  if (p!=NULL)
@@ -164,10 +172,13 @@ same:
      return Un_Atom_Check(atom,f2_word);
     }
 
- p=strrchr(f1,'/');
-#ifdef __msdos__
- if (p==NULL)
-     p=strrchr(f1,'\\');
+ p=strrchr(f1,DIR_SEP_C);
+#ifdef M_ix86_win32
+{
+ char *q=strrchr(f1,'/');
+ if (p==NULL || p>q)
+     p=q;
+}
 #endif
 
  if (strchr((p) ? p : f1,'.'))
