@@ -6,7 +6,7 @@
 /* Descr.: Prolog flag and system variable management - C Part             */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -24,16 +24,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
-#include "engine_pl.h"
+#define OBJ_INIT Flag_Initializer
 
 #define FLAG_C_FILE
 
+#include "engine_pl.h"
 #include "bips_pl.h"
 
-
+#ifndef M_ix86_win32
+#include <unistd.h>
 extern char **environ;
+#endif
+
 
 
 
@@ -87,8 +90,6 @@ static int atom_prolog[4];
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Flag_Initializer)
-
 static
 Bool      Unif_Flag             (int i,WamWord value_word);
 
@@ -102,9 +103,6 @@ Prolog_Prototype(ENVIRON_ALT,0)
 
 
 
-
-
-
 /*-------------------------------------------------------------------------*/
 /* FLAG_INITIALIZER                                                        */
 /*                                                                         */
@@ -308,7 +306,7 @@ Bool Set_Prolog_Flag_2(WamWord flag_word,WamWord value_word)
  return TRUE;
 
 err_value:
- term=Put_Structure(atom_char['+'],2);
+ term=Put_Structure(ATOM_CHAR('+'),2);
  Unify_Value(flag_word);
  Unify_Value(value_word);
 
@@ -655,7 +653,7 @@ void Sys_Var_Put_2(WamWord var_word,WamWord term_word)
 
  size=Term_Size(word);
 
- adr=(WamWord *) Malloc(size*sizeof(WamWord));
+ adr=(WamWord *) Malloc(size*sizeof(WamWord));          /* never recovered */
 
  Copy_Term(adr,&word);
 
