@@ -6,7 +6,7 @@
 /* Descr.: global variable (inline) management - C part                    */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -23,6 +23,8 @@
 /*-------------------------------------------------------------------------*/
 #include <stdlib.h>
 
+#define OBJ_INIT G_Var_Initializer
+
 #include "engine_pl.h"
 #include "bips_pl.h"
 
@@ -67,8 +69,6 @@ static int atom_g_array_extend;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(G_Var_Initializer)
-
 static 
 Bool      G_Assign              (WamWord gvar_word,WamWord gval_word,
                                  Bool backtrack,Bool copy);
@@ -384,7 +384,7 @@ deref:
 
      index=UnTag_INT(word);
 
-     if (size>=0 || (unsigned) index>= -size)
+     if (size>=0 || (unsigned) index>= (unsigned) (-size))
          Pl_Err_Domain(domain_g_array_index,gvar_word);
 
      g_elem=(GVarElt *) (g_elem->val)+index;
