@@ -6,7 +6,7 @@
 /* Descr.: operating system interface management                           */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -116,6 +116,10 @@ file_property(PathName,Property):-
 
 '$check_file_prop'(permission(_)).
 
+'$check_file_prop'(creation(_)).
+
+'$check_file_prop'(last_access(_)).
+
 '$check_file_prop'(last_modification(_)).
 
 '$check_file_prop'(Property):-
@@ -141,8 +145,17 @@ file_property(PathName,Property):-
 	'$call_c_test'('Check_Prop_Perm_And_File_2'(Perm,PathName)),
 	'$file_prop_perm'(Perm,PathName).
 
+'$file_prop'(creation(DateTime),PathName):-
+	'$sys_var_write'(0,0),
+	'$call_c_test'('File_Prop_Date_2'(DateTime,PathName)).
+
+'$file_prop'(last_access(DateTime),PathName):-
+	'$sys_var_write'(0,1),
+	'$call_c_test'('File_Prop_Date_2'(DateTime,PathName)).
+
 '$file_prop'(last_modification(DateTime),PathName):-
-	'$call_c_test'('File_Prop_Last_Modification_2'(DateTime,PathName)).
+	'$sys_var_write'(0,2),
+	'$call_c_test'('File_Prop_Date_2'(DateTime,PathName)).
 
 
 '$file_prop_perm'(read,PathName):-
@@ -302,7 +315,7 @@ prolog_pid(PrologPid):-
 
 
 send_signal(Pid,Signal):-
-	set_bip_name(kill,2),
+	set_bip_name(send_signal,2),
 	'$call_c_test'('Send_Signal_2'(Pid,Signal)).
 
 
