@@ -6,7 +6,7 @@
 /* Descr.: operating system interface management - C part                  */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -21,20 +21,32 @@
 /* with this program; if not, write to the Free Software Foundation, Inc.  */
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
+#include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#include <dirent.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <signal.h>
 #include <time.h>
-#include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+
+#include "gp_config.h"
+
+#ifdef M_ix86_win32
+#include <process.h>
+#include <direct.h>
+#include <io.h>
+#include <winsock.h>
+#else
+#include <dirent.h>
+#include <unistd.h>
+#include <sys/param.h>
 #include <sys/time.h>
-#include <sys/utsname.h>
 #include <sys/wait.h>
+#endif
+
+#define OBJ_INIT Os_Interf_Initializer
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -45,6 +57,8 @@
 #endif
 
 
+
+
 /*---------------------------------*/
 /* Constants                       */
 /*---------------------------------*/
@@ -97,8 +111,6 @@ static int    nb_sig;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Os_Interf_Initializer)
-
 static
 int       Flag_Of_Permission    (WamWord perm_word,Bool is_a_directory);
 static
@@ -168,9 +180,13 @@ static void Os_Interf_Initializer(void)
  tsig[nb_sig].atom=Create_Atom("SIGVTALRM"); tsig[nb_sig++].sig=SIGVTALRM;
  tsig[nb_sig].atom=Create_Atom("SIGPROF");   tsig[nb_sig++].sig=SIGPROF;
  tsig[nb_sig].atom=Create_Atom("SIGWINCH");  tsig[nb_sig++].sig=SIGWINCH;
+#ifndef M_ix86_sco
  tsig[nb_sig].atom=Create_Atom("SIGIO");     tsig[nb_sig++].sig=SIGIO;
+#endif
+#ifndef M_ix86_bsd
  tsig[nb_sig].atom=Create_Atom("SIGPOLL");   tsig[nb_sig++].sig=SIGPOLL;
 #endif
+#endif
 
 
 #if defined(__unix__) || defined(__CYGWIN__)
@@ -192,7 +208,11 @@ Bool Make_Directory_1(WamWord path_name_word)
 
  path_name=Get_Path_Name(path_name_word);
 
+#ifdef M_ix86_win32
+ Os_Test_Error(_mkdir(path_name))
+#else
  Os_Test_Error(mkdir(path_name,0777))
+#endif
 
  return TRUE;
 }
@@ -262,27 +282,65 @@ Bool Change_Directory_1(WamWord path_name_word)
 Bool Directory_Files_2(WamWord path_name_word,WamWord list_word)
 
 {
- char          *path_name;
- DIR           *dir;
- struct dirent *cur_entry;
+ char               *path_name;
+ Bool                res;
+ char               *name;
+#ifdef M_ix86_win32
+ long                h;
+ struct _finddata_t  d;
+ static
+ char                buff[MAXPATHLEN];
+#else
+ DIR                *dir;
+ struct dirent      *cur_entry;
+#endif
+
 
  Check_For_Un_List(list_word);
 
  path_name=Get_Path_Name(path_name_word);
 
+#ifdef M_ix86_win32
+ sprintf(buff,"%s\\*.*",path_name);
+ h=_findfirst(buff,&d); /* instead of Win32 FindFirstFile since uses errno */
+ Os_Test_Error(h== -1)
+#else
  dir=opendir(path_name);
  Os_Test_Error(dir==NULL)
+#endif
 
+#ifdef M_ix86_win32
+ do
+    {
+     name=d.name;
+#else
  while((cur_entry=readdir(dir))!=NULL)
     {
+     name=cur_entry->d_name;
+#endif
      if (!Get_List(list_word) ||
-         !Unify_Atom(Create_Allocate_Atom(cur_entry->d_name)))
-         return FALSE;
+         !Unify_Atom(Create_Allocate_Atom(name)))
+        {
+         res=FALSE;
+         goto finish;
+        }
 
      list_word=Unify_Variable();
     }
+#ifdef M_ix86_win32
+ while(_findnext(h,&d)==0);
+#endif
+ 
+ res=Get_Nil(list_word);
 
- return Get_Nil(list_word);
+finish:
+#ifdef M_ix86_win32
+ _findclose(h);
+#else
+ closedir(dir);
+#endif
+
+ return res;
 }
 
 
@@ -381,18 +439,24 @@ Bool File_Permission_2(WamWord path_name_word,WamWord perm_list_word)
  WamWord      save_perm_list_word;
  WamWord     *lst_adr;
  char        *path_name;
- int          perm=F_OK;
+ int          mode,perm=0;
  struct stat  file_info;
- Bool         is_a_directory=FALSE;
+ int          res;
+ Bool         is_a_directory;
 
  path_name=Get_Path_Name(path_name_word);
 
- if (stat(path_name,&file_info)==0)
-     is_a_directory=S_ISDIR(file_info.st_mode);
+ res=stat(path_name,&file_info);
+ if (res==-1 && errno!=ENOENT && errno!=ENOTDIR)
+     Os_Test_Error(1)
+
+ mode=file_info.st_mode;
+
+ is_a_directory=(res==0) && S_ISDIR(mode);
 
  Deref(perm_list_word,word,tag,adr)
  if (tag==ATM && word!=NIL_WORD)
-     perm|=Flag_Of_Permission(word,S_ISDIR(file_info.st_mode));
+     perm|=Flag_Of_Permission(word,is_a_directory);
   else
     {
      save_perm_list_word=perm_list_word;
@@ -411,23 +475,13 @@ Bool File_Permission_2(WamWord path_name_word,WamWord perm_list_word)
              Pl_Err_Type(type_list,save_perm_list_word);
 
          lst_adr=UnTag_LST(word);
-         perm|=Flag_Of_Permission(Car(lst_adr),S_ISDIR(file_info.st_mode));
+         perm|=Flag_Of_Permission(Car(lst_adr),is_a_directory);
 
          perm_list_word=Cdr(lst_adr);
         }
     }
 
- if (access(path_name,perm))
-    {
-     if (errno==ENOENT || errno==ENOTDIR ||     /* to allow existence test */
-         errno==EINVAL ||               /* for -1 given for search/execute */
-         errno==EACCES)                        /* for [RWX]_OK not allowed */
-         return FALSE;
-
-     Os_Test_Error(1)
-    }
-
- return TRUE;
+ return (res==0) && perm>0 && ((mode | perm) == mode);
 }
 
 
@@ -445,16 +499,16 @@ static int Flag_Of_Permission(WamWord perm_word,Bool is_a_directory)
  atom=Rd_Atom_Check(perm_word);
 
  if (atom==atom_read)
-     return R_OK;
+     return S_IRUSR;
 
  if (atom==atom_write)
-     return W_OK;
+     return S_IWUSR;
 
  if (atom==atom_execute)
-     return (is_a_directory) ? -1 : X_OK;
+     return (is_a_directory) ? -1 : S_IXUSR;
 
  if (atom==atom_search)
-     return (is_a_directory) ? X_OK : -1;
+     return (is_a_directory) ? S_IXUSR : -1;
 
  Pl_Err_Domain(domain_os_file_permission,perm_word);
  return 0;                                    /* anything for the compiler */
@@ -491,12 +545,15 @@ Bool File_Prop_Real_File_Name_2(WamWord real_path_name_word,
                                 WamWord path_name_word)
 
 {
- char *path_name;
+ char *path_name=Get_Path_Name(path_name_word);
+#ifndef M_ix86_win32
  char  real_path_name[MAXPATHLEN];
 
- path_name=Get_Path_Name(path_name_word);
-
  Os_Test_Error(realpath(path_name,real_path_name)==NULL)
+#else
+ char *real_path_name=path_name;
+#endif
+
  return Un_String_Check(real_path_name,real_path_name_word);
 }
 
@@ -597,21 +654,36 @@ Bool Check_Prop_Perm_And_File_2(WamWord perm_word,WamWord path_name_word)
 
 
 /*-------------------------------------------------------------------------*/
-/* FILE_PROP_LAST_MODIFICATION_2                                           */
+/* FILE_PROP_DATE_2                                                        */
 /*                                                                         */
 /*-------------------------------------------------------------------------*/
-Bool File_Prop_Last_Modification_2(WamWord date_time_word,
-                                   WamWord path_name_word)
+Bool File_Prop_Date_2(WamWord date_time_word,WamWord path_name_word)
 
 {
  char        *path_name;
  struct stat  file_info;
+ time_t      *t;
 
  path_name=Get_Path_Name(path_name_word);
 
  Os_Test_Error(stat(path_name,&file_info))
 
- return Date_Time_To_Prolog(&(file_info.st_mtime),date_time_word);
+ switch(sys_var[0])
+    {
+     case 0:
+         t=&(file_info.st_ctime);
+         break;
+
+     case 1:
+         t=&(file_info.st_atime);
+         break;
+
+     default:
+         t=&(file_info.st_mtime);
+         break;
+    }
+
+ return Date_Time_To_Prolog(t,date_time_word);
 }
 
 
@@ -624,8 +696,8 @@ Bool File_Prop_Last_Modification_2(WamWord date_time_word,
 Bool Temporary_Name_2(WamWord template_word,WamWord path_name_word)
 
 {
- char    *template;
- char    *path_name;
+ char *template;
+ char *path_name;
 
  template=Get_Path_Name(template_word);
 
@@ -697,6 +769,12 @@ Bool Host_Name_1(WamWord host_name_word)
 {
  WamWord word,tag,*adr;
  int     atom;
+ static
+ int     atom_host_name= -1;    /* not created in an init since implies to */
+                             /* establish a connection (if !NO_USE_SOCKETS)*/
+
+ if (atom_host_name<0)
+     atom_host_name=Create_Allocate_Atom(M_Host_Name_From_Name(NULL));
 
  Deref(host_name_word,word,tag,adr)
  if (tag==REF)
@@ -719,14 +797,7 @@ Bool Host_Name_1(WamWord host_name_word)
 Bool Os_Version_1(WamWord os_version_word)
 
 {
- struct utsname uname_info;
- char           os_version[4096];
-
- Os_Test_Error(uname(&uname_info)<0)
-
- sprintf(os_version,"%s %s",uname_info.sysname,uname_info.release);
-
- return Un_String_Check(os_version,os_version_word);
+ return Un_String_Check(m_os_version,os_version_word);
 }
 
 
@@ -739,11 +810,42 @@ Bool Os_Version_1(WamWord os_version_word)
 Bool Architecture_1(WamWord architecture_word)
 
 {
- struct utsname uname_info;
+ return Un_String_Check(m_architecture,architecture_word);
+}
 
- Os_Test_Error(uname(&uname_info)<0)
 
- return Un_String_Check(uname_info.machine,architecture_word);
+
+
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* SLEEP_1                                                                 */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+void Sleep_1(WamWord seconds_word)
+
+{
+#ifdef M_ix86_win32
+ DWORD ms;
+
+ ms=(DWORD) (Rd_Number_Check(seconds_word)*1000);
+
+ if (ms<0)
+     Pl_Err_Domain(domain_not_less_than_zero,seconds_word);
+
+ Sleep(ms);
+#else
+ long us;
+
+ us=(long) (Rd_Number_Check(seconds_word)*1000000);
+
+ if (us<0)
+     Pl_Err_Domain(domain_not_less_than_zero,seconds_word);
+
+ usleep(us);
+#endif
 }
 
 
@@ -764,6 +866,7 @@ Bool Shell_2(WamWord cmd_word,WamWord status_word)
      cmd=NULL;
  Check_For_Un_Integer(status_word);
 
+ Flush_All_Streams();
  status=M_Shell(cmd);
 
  return Get_Integer(status,status_word);
@@ -785,6 +888,11 @@ Bool System_2(WamWord cmd_word,WamWord status_word)
  cmd=Rd_String_Check(cmd_word);
  Check_For_Un_Integer(status_word);
 
+#ifdef M_ix86_win32
+ _flushall();
+#endif
+
+ Flush_All_Streams();
  status=system(cmd);
 
  return Get_Integer(status,status_word);
@@ -803,17 +911,14 @@ Bool Spawn_3(WamWord cmd_word,WamWord list_word,WamWord status_word)
  WamWord   word,tag,*adr;
  WamWord   save_list_word;
  WamWord  *lst_adr;
- char     *cmd;
- int       status;
  char     *arg[MAX_SPAWN_ARGS];
  char    **p=arg;
  char      err[64];
-
- cmd=Rd_String_Check(cmd_word);
+ int       status;
 
  save_list_word=list_word;
 
- *p++=cmd;
+ *p++=Rd_String_Check(cmd_word);
 
  for(;;)
     {
@@ -838,13 +943,15 @@ Bool Spawn_3(WamWord cmd_word,WamWord list_word,WamWord status_word)
  *p=NULL;
  Check_For_Un_Integer(status_word);
 
- status=M_Spawn(cmd,arg);
+ Flush_All_Streams();
+ status=M_Spawn(arg);
 
- Os_Test_Error(status<-1)
- if (status== -1)
+ Os_Test_Error(status== -1)
+ if (status== -2)
     {
-     sprintf(err,"error trying to execute %s",cmd);
+     sprintf(err,"error trying to execute %s",arg[0]);
      Pl_Err_System(Create_Allocate_Atom(err));
+     return FALSE;
     }
 
  return Get_Integer(status,status_word);
@@ -853,26 +960,6 @@ Bool Spawn_3(WamWord cmd_word,WamWord list_word,WamWord status_word)
 
 
 
-/*-------------------------------------------------------------------------*/
-/* SLEEP_1                                                                 */
-/*                                                                         */
-/*-------------------------------------------------------------------------*/
-void Sleep_1(WamWord seconds_word)
-
-{
- long us;
-
- us=(long) (Rd_Number_Check(seconds_word)*1000000);
-
- if (us<0)
-     Pl_Err_Domain(domain_not_less_than_zero,seconds_word);
-
- usleep(us);
-}
-
-
-
-
 /*-------------------------------------------------------------------------*/
 /* POPEN_3                                                                 */
 /*                                                                         */
@@ -898,13 +985,14 @@ Bool Popen_3(WamWord cmd_word,WamWord mode_word,WamWord stm_word)
  prop.special_close=FALSE;
  prop.other        =0;
 
+ Flush_All_Streams();
  f=popen(cmd,open_str);
  Os_Test_Error(f==NULL)
 
  sprintf(stream_name,"popen_stream('%.*s')",
          (int) sizeof(stream_name)-20,cmd);
  stm=Add_Stream(Create_Allocate_Atom(stream_name),(long) f,prop,
-                NULL,NULL,NULL,pclose,NULL,NULL,NULL);
+                NULL,NULL,NULL,(StmFct) pclose,NULL,NULL,NULL);
 
  return Get_Integer(stm,stm_word);
 }
@@ -921,78 +1009,32 @@ Bool Exec_5(WamWord cmd_word,WamWord stm_in_word,WamWord stm_out_word,
 
 {
  char     *cmd;
+ char    **arg;
  StmProp   prop;
  int       stm;
  FILE     *f_in,*f_out,*f_err;
- int       pipe_in[2],pipe_out[2],pipe_err[2];
  int       pid;
  int       mask=SYS_VAR_OPTION_MASK;
  char      stream_name[512];
  int       atom;
-
+ char      err[64];
 
  cmd=Rd_String_Check(cmd_word);
-
- Os_Test_Error(pipe(pipe_in))
- Os_Test_Error(pipe(pipe_out))
- Os_Test_Error(pipe(pipe_err))
+ arg=M_Create_Shell_Command(cmd);
 
  Flush_All_Streams();
- pid=(int) fork();
+ pid=M_Spawn_Redirect(arg,(mask & 1)==0,&f_in,&f_out,&f_err);
  Os_Test_Error(pid== -1)
+ if (pid== -2)
+    {
+     sprintf(err,"error trying to execute %s",cmd);
+     Pl_Err_System(Create_Allocate_Atom(err));
+     return FALSE;
+    }
 
  if (mask & 1)                                             /* pid needed ? */
      Get_Integer(pid,pid_word);
 
- if (pid==0)                                          /* the child process */
-    {
-     if ((mask & 1) || fork()==0)                          /* pid needed ? */
-        {          /* if pid not needed nested fork to detach exec process */
-         close(pipe_in[1]);                     /* to avoid zombie process */
-         if (pipe_in[0]!=0)
-            {
-             Os_Test_Error(dup2(pipe_in[0],0)== -1)
-             Os_Test_Error(close(pipe_in[0]))
-            }
-         close(pipe_out[0]);
-         if (pipe_out[1]!=1)
-            {
-             Os_Test_Error(dup2(pipe_out[1],1)== -1)
-             Os_Test_Error(close(pipe_out[1]))
-            }
-    
-         close(pipe_err[0]);
-         if (pipe_err[1]!=2)
-            {
-             Os_Test_Error(dup2(pipe_err[1],2)== -1)
-             Os_Test_Error(close(pipe_err[1]))
-            }
-    
-         execlp("sh","sh","-c",cmd,NULL);         /* only returns on error */
-         Os_Test_Error(1)
-         Flush_All_Streams();
-         exit(1);
-        }
-      else
-         exit(0);                        /* pid not needed terminate child */
-    }
-
- if ((mask & 1)==0)                                    /* pid not needed ? */
-     waitpid(pid,NULL,0);                        /* wait child termination */
-
-
- Os_Test_Error(close(pipe_in[0]))
- f_in=fdopen(pipe_in[1],"wt");
- Os_Test_Error(f_in==NULL)
-
- Os_Test_Error(close(pipe_out[1]))
- f_out=fdopen(pipe_out[0],"rt");
- Os_Test_Error(f_out==NULL)
-
- Os_Test_Error(close(pipe_err[1]))
- f_err=fdopen(pipe_err[0],"rt");
- Os_Test_Error(f_err==NULL)
-
  sprintf(stream_name,"exec_stream('%.*s')",
          (int) sizeof(stream_name)-20,cmd);
  atom=Create_Allocate_Atom(stream_name);
@@ -1043,6 +1085,13 @@ Bool Select_5(WamWord reads_word,WamWord ready_reads_word,
               WamWord time_out_word)
 
 {
+#if defined(M_ix86_win32) && defined(NO_USE_SOCKETS)
+
+ Pl_Err_Resource(Create_Atom("not implemented"));
+ return FALSE;
+
+#else
+
  double          time_out;
  struct timeval *p,t;
  fd_set          read_set,write_set;
@@ -1062,8 +1111,8 @@ Bool Select_5(WamWord reads_word,WamWord ready_reads_word,
      p=NULL;
   else
     {
-     t.tv_sec =time_out/1000;
-     t.tv_usec=fmod(time_out,1000)*1000;
+     t.tv_sec =(long) (time_out/1000);
+     t.tv_usec=(long) (fmod(time_out,1000)*1000);
      p=&t;
     }
 
@@ -1073,6 +1122,8 @@ Bool Select_5(WamWord reads_word,WamWord ready_reads_word,
 
  return Select_Init_Ready_List(reads_word,&read_set,ready_reads_word) &&
         Select_Init_Ready_List(writes_word,&write_set,ready_writes_word);
+
+#endif
 }
 
 
@@ -1234,7 +1285,26 @@ Bool Send_Signal_2(WamWord pid_word,WamWord signal_word)
   else
      sig=Rd_Integer_Check(word);
 
+#ifdef M_ix86_win32
+{
+ int ret;
+
+ if (pid!=_getpid())
+    {
+     errno=EINVAL;
+     ret=1;
+    }
+  else
+    {
+     errno=-1;
+     ret=raise(sig);
+    }
+
+ Os_Test_Error(ret)
+}
+#else
  Os_Test_Error(kill(pid,sig))
+#endif
 
  return TRUE;
 }
@@ -1255,10 +1325,14 @@ Bool Wait_2(WamWord pid_word,WamWord status_word)
  pid=Rd_Integer_Check(pid_word);
  Check_For_Un_Integer(status_word);
 
+#ifdef M_ix86_win32
+ Os_Test_Error(_cwait(&status,pid,_WAIT_CHILD)== -1)
+#else
  Os_Test_Error(waitpid(pid,&status,0)== -1)
 
  if (WIFEXITED(status))
      status=WEXITSTATUS(status);
+#endif
 
  return Get_Integer(status,status_word);
 }
