@@ -6,7 +6,7 @@
 /* Descr.: parser support                                                  */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -26,6 +26,8 @@
 #include <ctype.h>
 #include <setjmp.h>
 
+#define OBJ_INIT Parse_Supp_Initializer
+
 #define PARSE_SUPP_FILE
 
 #include "engine_pl.h"
@@ -61,7 +63,7 @@ static StmInf  *pstm_i;
 static Bool     tok_present;
 
 static jmp_buf  jumper;
-#ifndef NO_USE_REGS
+#if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS>0
 static WamWord  buff_save_machine_regs[NB_OF_USED_MACHINE_REGS];
 #endif
 
@@ -80,9 +82,6 @@ static int atom_extend;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Parse_Supp_Initializer)
-
-
 static 
 void      Read_Next_Token       (Bool comma_is_punct);
 
@@ -307,7 +306,7 @@ static WamWord Parse_Term(int cur_prec,int context,Bool comma_is_punct)
              if (flag_value==FLAG_DOUBLE_QUOTES_CODES)
                  Unify_Integer(token.name[i]);
               else
-                 Unify_Atom(atom_char[(unsigned char) (token.name[i])]);
+                 Unify_Atom(ATOM_CHAR(token.name[i]));
 
              Unify_Value(term);
              term=term1;
@@ -764,7 +763,7 @@ arg_of_struct:
          token.punct='(';               /* and then like TOKEN_PUNCTUATION */
      case TOKEN_PUNCTUATION:
 	 func=atom_punct;
-         atom=atom_char[token.punct];
+         atom=ATOM_CHAR(token.punct);
          arg=Put_Atom(atom);
          break;
 
