@@ -6,7 +6,7 @@
 /* Descr.: pretty print clause management - C part                         */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,6 +22,8 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 
+#define OBJ_INIT Pretty_Initializer
+
 #include "engine_pl.h"
 #include "bips_pl.h"
 
@@ -77,8 +79,6 @@ static int       nb_to_try;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Pretty_Initializer)
-
 static
 void      Portray_Clause        (StmInf *pstm,WamWord term_word);
 
@@ -128,7 +128,7 @@ static void Pretty_Initializer(void)
 
  dollar_var_1       =Functor_Arity(atom_dollar_var,1);
  dollar_varname_1   =Functor_Arity(atom_dollar_varname,1);
- equal_2            =Functor_Arity(atom_char['='],2);
+ equal_2            =Functor_Arity(ATOM_CHAR('='),2);
 }
 
 
@@ -269,11 +269,11 @@ static Bool Is_Cut(WamWord body_word)
  WamWord word,tag,*adr;
  WamWord arg_word[2];
 
- while(Check_Structure(body_word,atom_char[','],2,arg_word))
+ while(Check_Structure(body_word,ATOM_CHAR(','),2,arg_word))
      body_word=arg_word[0];
 
  Deref(body_word,word,tag,adr)
- return (tag==ATM && UnTag_ATM(word)==atom_char['!']);
+ return (tag==ATM && UnTag_ATM(word)==ATOM_CHAR('!'));
 }
 
 
@@ -292,7 +292,7 @@ static void Show_Body(StmInf *pstm,int level,int context,
  int     prec[]={1200-1 , 1000-1,1000 , 1100-1,1100 , 1050-1,1050 };
 
 
- if (Check_Structure(body_word,atom_char[','],2,arg_word))
+ if (Check_Structure(body_word,ATOM_CHAR(','),2,arg_word))
     {
      Show_Body(pstm,level,LEFT_AND,arg_word[0]);
      Stream_Putc(',',pstm);
@@ -305,7 +305,7 @@ static void Show_Body(StmInf *pstm,int level,int context,
     }
 
 
- if (Check_Structure(body_word,atom_char[';'],2,arg_word))
+ if (Check_Structure(body_word,ATOM_CHAR(';'),2,arg_word))
     {
      if (context!=RIGHT_OR)
         {
@@ -406,7 +406,7 @@ void Name_Singleton_Vars_1(WamWord start_word)
      return;
 
  word=Put_Structure(atom_dollar_varname,1);
- Unify_Atom(atom_char['_']);                    /* bind to '$VARNAME'('_') */
+ Unify_Atom(ATOM_CHAR('_'));                    /* bind to '$VARNAME'('_') */
 
  while(--singl_var_ptr>=glob_dico_var)
     {
