@@ -6,7 +6,7 @@
 /* Descr.: random number generator management - C part                     */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -106,8 +106,8 @@ Bool Random_3(WamWord l_word,WamWord u_word,WamWord n_word)
  if (l>=u)
      return FALSE;
 
- l1=l;
- u1=u;
+ l1=(long) l;
+ u1=(long) u;
 
  if (l1==l && u1==u)
     {
