@@ -6,7 +6,7 @@
 /* Descr.: read/1 and friends - C part                                     */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -138,7 +138,7 @@ Bool Read_Term_5(WamWord sora_word,WamWord term_word,
                                   /* glob_dico_var: variable names (atoms) */
          glob_dico_var[i]=Create_Allocate_Atom(parse_dico_var[i].name);
 
-         word=Put_Structure(atom_char['='],2);
+         word=Put_Structure(ATOM_CHAR('='),2);
          Unify_Atom(glob_dico_var[i]);
          Unify_Value(parse_dico_var[i].word);
 
@@ -164,7 +164,7 @@ Bool Read_Term_5(WamWord sora_word,WamWord term_word,
          if ((SYS_VAR_OPTION_MASK & 2)==0)            /* not yet allocated */
              glob_dico_var[i]=Create_Allocate_Atom(parse_dico_var[i].name);
 
-         word=Put_Structure(atom_char['='],2);
+         word=Put_Structure(ATOM_CHAR('='),2);
          Unify_Atom(glob_dico_var[i]);
          Unify_Value(parse_dico_var[i].word);
 
@@ -415,8 +415,8 @@ Bool Current_Char_Conversion_2(WamWord in_char_word,WamWord out_char_word)
                          Prolog_Predicate(CURRENT_CHAR_CONVERSION_ALT,0),4);
     }
 
- return Get_Atom(atom_char[c_in], in_char_word) &&
-        Get_Atom(atom_char[c_out],out_char_word);
+ return Get_Atom(ATOM_CHAR(c_in), in_char_word) &&
+        Get_Atom(ATOM_CHAR(c_out),out_char_word);
 }
 
 
@@ -455,6 +455,6 @@ Bool Current_Char_Conversion_Alt_0(void)
      AB(B,3)=c_out1;
     }
 
- return Get_Atom(atom_char[c_in], in_char_word) &&
-        Get_Atom(atom_char[c_out],out_char_word);
+ return Get_Atom(ATOM_CHAR(c_in), in_char_word) &&
+        Get_Atom(ATOM_CHAR(c_out),out_char_word);
 }
