@@ -6,7 +6,7 @@
 /* Descr.: stream selection and control management - C part                */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -25,12 +25,20 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-#include <unistd.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 
 #include "engine_pl.h"
 #include "bips_pl.h"
 
+#if defined(M_ix86_cygwin) || defined(M_ix86_win32)
+#include <io.h>
+#endif
+
+#ifndef M_ix86_win32
+#include <unistd.h>
+#include <sys/fcntl.h>
+#endif
+
 
 
 
@@ -219,7 +227,7 @@ void Open_3(WamWord source_sink_word,WamWord mode_word,WamWord stm_word)
                            permission_type_source_sink,source_sink_word);
     }
 
- prop.tty=isatty(fileno(f));
+ prop.tty=(isatty(fileno(f))!=0);
 
  if ((mask & 2)==0)                      /* not specified - set to default */
      prop.reposition=!(prop.tty);
@@ -331,7 +339,7 @@ void Set_Stream_Type_2(WamWord sora_word,WamWord is_text_word)
  pstm=stm_tbl+stm;
 
  text=Rd_Integer_Check(is_text_word);
- if (text==pstm->prop.text)
+ if ((unsigned) text==pstm->prop.text)
      return;
 
  if (pstm->char_count)
@@ -340,16 +348,15 @@ void Set_Stream_Type_2(WamWord sora_word,WamWord is_text_word)
                        sora_word);
 
  pstm->prop.text=text;
-#ifdef M_ix86_cygwin
+#if defined(M_ix86_cygwin) || defined(M_ix86_win32)
 {
  FILE *f;
- int setmode(int nHandle,int nAccessMode);
 
  f=File_Star_Of_Stream(stm);
  if (f==NULL)
-     return ;
+     return;
 
- setmode (fileno(f), (text) ? O_TEXT : O_BINARY);
+ setmode(fileno(f), (text) ? O_TEXT : O_BINARY);
 }
 #endif
 }
