@@ -6,7 +6,7 @@
 /* Descr.: stream support                                                  */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -23,18 +23,21 @@
 /*-------------------------------------------------------------------------*/
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
+#include <string.h>
 #include <stdarg.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
+
 #ifdef clearerr             /* prevent the case clearerr is (also) a macro */
 #undef clearerr
 extern void clearerr(FILE *stream);
 #endif
 
 
+#define OBJ_INIT Stream_Supp_Initializer
+
 #define STREAM_SUPP_FILE
 
 #include "engine_pl.h"
@@ -44,6 +47,12 @@ extern void clearerr(FILE *stream);
 #include "linedit.h"
 #endif
 
+#ifdef M_ix86_win32
+#include <io.h>
+#else
+#include <unistd.h>
+#endif
+
 
 
 
@@ -55,7 +64,7 @@ extern void clearerr(FILE *stream);
 
 #define STR_STREAM_WRITE_BLOCK     1024
 
-#define TTY_BUFFER_SIZE            256
+#define TTY_BUFFER_SIZE            1024
 
 #define BIG_BUFFER                 2048
 
@@ -74,7 +83,6 @@ extern void clearerr(FILE *stream);
 /*---------------------------------*/
 
 #ifndef NO_USE_LINEDIT
-
 typedef struct                           /* TTY input stream information   */
     {                                    /* ------------------------------ */
      FILE    *f;                         /* associated input stream        */
@@ -83,7 +91,6 @@ typedef struct                           /* TTY input stream information   */
      char     buff[TTY_BUFFER_SIZE];     /* current buffer (end with '\0') */
      char    *ptr;                       /* current pointer into the buffer*/
     }TTYInf;
-
 #endif
 
 
@@ -108,8 +115,6 @@ static StrSInf  glob_str_stream={NULL,NULL,0};
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Stream_Supp_Initializer)
-
 static
 int       Find_Free_Stream      (void);
 
@@ -127,14 +132,12 @@ static
 int       TTY_Close             (TTYInf *tty);
 static
 void      TTY_Clearerr          (TTYInf *tty);
+#endif
 
-#else
-#define   TTY_Getc              NULL
-#define   TTY_Getc_No_Echo      NULL
-#define   TTY_Putc              NULL
-#define   TTY_Flush             NULL
-#define   TTY_Close             NULL
-#define   TTY_Clearerr          NULL
+
+#ifdef W32_GUI_CONSOLE
+static
+void      W32_Putc              (int c,long file);
 #endif
 
 static 
@@ -209,15 +212,24 @@ static void Stream_Supp_Initializer(void)
  atom_eof             =Create_Atom("eof");
 
 
+#ifdef W32_GUI_CONSOLE
+ if (w32gc_present)                              /* linked with W32GUICons */
+     istty=1;
+  else
+#endif
+     istty=(isatty(fileno(stdin))!=0);
 
- istty=isatty(fileno(stdin));
  prop.mode         =STREAM_MODE_READ;
  prop.input        =TRUE;
  prop.output       =FALSE;
  prop.text         =TRUE;
  prop.reposition   =FALSE;
  prop.eof_action   =STREAM_EOF_ACTION_RESET;
+#ifdef M_ix86_win32 /* MSVC++ doesn't detect a tty on stdin/out under RXVT */
+ prop.buffering    =STREAM_BUFFERING_NONE;
+#else
  prop.buffering    =(istty) ? STREAM_BUFFERING_LINE : STREAM_BUFFERING_BLOCK;
+#endif
  prop.tty          =istty;
  prop.special_close=FALSE;
  prop.other        =0;
@@ -229,14 +241,24 @@ static void Stream_Supp_Initializer(void)
 
 
 
- istty=isatty(fileno(stdout));
+#ifdef W32_GUI_CONSOLE
+ if (w32gc_present)                              /* linked with W32GUICons */
+     istty=1;
+  else
+#endif
+     istty=(isatty(fileno(stdout))!=0);
+
  prop.mode         =STREAM_MODE_APPEND;
  prop.input        =FALSE;
  prop.output       =TRUE;
  prop.text         =TRUE;
  prop.reposition   =FALSE;
  prop.eof_action   =STREAM_EOF_ACTION_RESET;
+#ifdef M_ix86_win32 /* MSVC++ doesn't detect a tty on stdin/out under RXVT */
+ prop.buffering    =STREAM_BUFFERING_NONE;
+#else
  prop.buffering    =(istty) ? STREAM_BUFFERING_LINE : STREAM_BUFFERING_BLOCK;
+#endif
  prop.tty          =istty;
  prop.special_close=FALSE;
  prop.other        =0;
@@ -322,22 +344,34 @@ int Add_Stream(int atom_file_name,long file,StmProp prop,
 {
  int     stm;
  StmInf *pstm;
-#ifndef NO_USE_LINEDIT
- TTYInf *tty;
- char   *p;
-#endif
  int     d;
  static
- StmFct  def[2][7]=
-         { { (StmFct) fgetc,       (StmFct) fputc,
+ StmFct  def[3][7]=
+         { { (StmFct)
+#ifndef M_ix86_bsd
+ fgetc
+#else
+ getc
+#endif
+                           ,       (StmFct) fputc,
              (StmFct) fflush,      (StmFct) fclose,
              (StmFct) ftell,       (StmFct) fseek,
-             (StmFct) clearerr     } ,
+             (StmFct) clearerr     }
+#ifndef NO_USE_LINEDIT
+           ,
            { (StmFct) TTY_Getc,    (StmFct) TTY_Putc,
              (StmFct) TTY_Flush,   (StmFct) TTY_Close,
              STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
-             (StmFct) TTY_Clearerr } };
-
+             (StmFct) TTY_Clearerr }
+#endif
+#ifdef W32_GUI_CONSOLE
+           ,
+           { (StmFct) NULL,        (StmFct) W32_Putc,
+             STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
+             STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
+             STREAM_FCT_UNDEFINED }
+#endif
+           };
 
 
  stm=Find_Free_Stream();
@@ -350,26 +384,43 @@ int Add_Stream(int atom_file_name,long file,StmProp prop,
  pstm=stm_tbl+stm;
 
  pstm->atom_file_name=atom_file_name;
-#ifndef NO_USE_LINEDIT
- if (prop.tty && prop.input)
-    {
-     tty=(TTYInf *) Malloc(sizeof(TTYInf));
+ pstm->file=file;
+ d=0;
 
-     pstm->file=(long) tty;
-     tty->f=(FILE *) file;
-     tty->fd_in =fileno((FILE *) file);
-     if ((p=ttyname(tty->fd_in))==NULL || (tty->fd_out=open(p,O_WRONLY))==-1)
-          tty->fd_out=1;
-     tty->buff[0]='\0';
-     tty->ptr=tty->buff;
-     d=1;
-    }
-  else
-#endif
+#ifndef NO_USE_LINEDIT
+ if (prop.tty)
     {
-     pstm->file=file;
-     d=0;
+     if (prop.input)
+        {
+         TTYInf *tty=(TTYInf *) Malloc(sizeof(TTYInf));
+
+         pstm->file=(long) tty;
+         tty->f=(FILE *) file;
+         tty->fd_in =fileno((FILE *) file);
+#if defined(M_ix86_win32)
+         tty->fd_out=1;
+#else
+        {
+         char *p;
+
+         if ((p=ttyname(tty->fd_in))==NULL || 
+             (tty->fd_out=open(p,O_WRONLY))==-1)
+              tty->fd_out=1;
+        }
+#endif
+         tty->buff[0]='\0';
+         tty->ptr=tty->buff;
+         d=1;
+        }
+#ifdef W32_GUI_CONSOLE
+      else
+         if (w32gc_present)
+            {
+             d=2;
+            }
+#endif /* W32_GUI_CONSOLE */
     }
+#endif /* NO_USE_LINEDIT */
 
  pstm->prop=prop;
 
@@ -580,7 +631,11 @@ void Set_Stream_Buffering(int stm)
          break;
 
      case STREAM_BUFFERING_LINE:
+#ifdef M_ix86_win32         /* in Win32 console app, line buff = full buff */
+         buff_flag=_IONBF;                          /* I prefer no buff... */
+#else
          buff_flag=_IOLBF;
+#endif
          break;
 
      case STREAM_BUFFERING_BLOCK:
@@ -588,7 +643,9 @@ void Set_Stream_Buffering(int stm)
          break;
     }
 
-#ifndef M_ix86_cygwin
+                               /* for those architectures we cannot modify */
+                               /* buffering when an I/O as already occured */
+#if !defined(M_ix86_bsd) && !defined(M_ix86_cygwin)
  setvbuf(f,NULL,buff_flag,BUFSIZ);
 #endif
 }
@@ -900,6 +957,34 @@ static void TTY_Clearerr(TTYInf *tty)
 
 
 
+/*-------------------------------------------------------------------------*/
+/* The following functions replaces standard fputc/... for the std output  */
+/* which is mapped to the W32 GUI consoleTTY.                              */
+/* These functions should not be used directly but via the common interface*/
+/* provided by the Stream_Putc/... functions (see below).                  */
+/*-------------------------------------------------------------------------*/
+
+#ifdef W32_GUI_CONSOLE
+
+
+/*-------------------------------------------------------------------------*/
+/* W32_PUTC                                                                */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static void W32_Putc(int c,long file)
+
+{
+ (*w32gc_put_char)(c);
+}
+
+
+
+
+#endif /* W32_GUI_CONSOLE */
+
+
+
+
 /*-------------------------------------------------------------------------*/
 /* Only the following functions should be used to read/write a stream.     */
 /*-------------------------------------------------------------------------*/
@@ -965,12 +1050,13 @@ int Stream_Getc_No_Echo(StmInf *pstm)
      c=TTY_Getc_No_Echo((TTYInf *) file);
 #else
      c=pstm->fct_getc((FILE *) file);
-     if (c!='\n')
-         while(pstm->fct_getc((FILE *) file)>=' ')
-             ;
-     if (pstm-stm_tbl==stm_stdin)
-         Update_Counters((stm_tbl+stm_stdout),'\n')
 
+ if (c!='\n')
+     while(pstm->fct_getc((FILE *) file)>=' ')
+         ;
+
+ if (pstm-stm_tbl==stm_stdin)
+     Update_Counters((stm_tbl+stm_stdout),'\n')
 #endif
 
  if (c==EOF)
@@ -997,7 +1083,6 @@ int Stream_Getc(StmInf *pstm)
  int  c;
  long file=pstm->file;
 
-
  Before_Reading(pstm,file)
 
  if (!PB_Is_Empty(pstm->pb_char))
