@@ -6,7 +6,7 @@
 /* Descr.: test - C part                                                   */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -21,9 +21,12 @@
 /* with this program; if not, write to the Free Software Foundation, Inc.  */
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
+#if 0
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#endif
+
 #if 1
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -80,7 +83,7 @@ Bool Test(WamWord w)
  arg[0]=Mk_Integer(12);
  arg[1]=Mk_Float(12.56);
  arg[2]=Mk_String("toto");
- return Un_Compound_Check(atom_char['f'],3,arg,w);
+ return Un_Compound_Check(ATOM_CHAR('f'),3,arg,w);
 
 #elif 0
 
@@ -90,7 +93,7 @@ Bool Test(WamWord w)
  arg[0]=Mk_Integer(12);
  arg[1]=Mk_Float(12.56);
  arg[2]=Mk_String("toto");
- r=Mk_Compound(atom_char['.'],2,arg);
+ r=Mk_Compound(ATOM_CHAR('.'),2,arg);
  return Unify(w,r);
 
 #elif 0
@@ -251,3 +254,5 @@ void Compile_Clause(WamWord head_word,WamWord body_word)
 
 {
 }
+
+
