@@ -6,7 +6,7 @@
 /* Descr.: term support                                                    */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -519,7 +519,7 @@ int Get_Pred_Indicator(WamWord pred_indic_word,Bool must_be_ground,
  if (tag==REF && must_be_ground)
      Pl_Err_Instantiation();
 
- if (!Get_Structure(atom_char['/'],2,pred_indic_word))
+ if (!Get_Structure(ATOM_CHAR('/'),2,pred_indic_word))
     {
      if (!Flag_Value(FLAG_STRICT_ISO) &&
          Rd_Callable(word,&func,arity)!=NULL)
