@@ -6,7 +6,7 @@
 /* Descr.: top Level - C part                                              */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -94,7 +94,9 @@ static void Ctrl_C_Handler(int sig)
  StmInf  *pstm=stm_tbl+stm_top_level_output;
  PredInf *pred;
  int      c;
+#ifndef M_ix86_win32
  sigset_t set;
+#endif
 
  signal(SIGINT, SIG_IGN);
  signal(SIGQUIT,SIG_IGN);
@@ -111,10 +113,12 @@ start:
  signal(SIGINT, Ctrl_C_Handler);
  signal(SIGQUIT,Ctrl_C_Handler);
 
+#ifndef M_ix86_win32
  sigemptyset(&set);
  sigaddset(&set,SIGINT);
  sigaddset(&set,SIGQUIT);
  sigprocmask(SIG_UNBLOCK,&set,NULL);
+#endif
 
  switch(c)
     {
