@@ -6,7 +6,7 @@
 /* Descr.: write term support                                              */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -24,6 +24,8 @@
 #include <string.h>
 #include <ctype.h>
 
+#define OBJ_INIT Write_Supp_Initializer
+
 #define WRITE_SUPP_FILE
 
 #include "engine_pl.h"
@@ -94,8 +96,6 @@ static int      last_prefix_op=W_NO_PREFIX_OP;
 /* Function Prototypes             */
 /*---------------------------------*/
 
-Declare_Object_Initializer(Write_Supp_Initializer)
-
 static
 void      Show_Term             (int depth,int prec,int context,
                                  WamWord term_word);
@@ -676,7 +676,7 @@ postfix:
 
      Show_Term(depth,oper->left,context,Arg(stc_adr,0));
 
-     if (functor==atom_char[','] && !ignore_op)
+     if (functor==ATOM_CHAR(',') && !ignore_op)
          Out_Char(',');
       else
         {
