@@ -6,7 +6,7 @@
 /* Descr.: FD instruction implementation                                   */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -387,7 +387,7 @@ void Fd_List_Int_To_Range(Range *range,WamWord list_word)
 
      val=UnTag_INT(word);
 
-     if ((unsigned) val>vec_max_integer)
+     if ((unsigned) val>(unsigned) vec_max_integer)
          range->extra_cstr=TRUE;
       else
 	{
@@ -856,16 +856,17 @@ start:
      goto do_propag;
     }
 
-
+                                     /* here if sparse OR n==min OR n==max */
  Trail_Range_If_Necessary(fdv_adr)
 
  propag=MASK_EMPTY;
  Set_Dom_Mask(propag)
+ if (Is_Sparse(r))
+     Vector_Reset_Value(r->vec,n);
 
- Range_Reset_Value(r,n);   /* must be before the update of r->min / r->max */
  Nb_Elem(fdv_adr)--;         
 
- if (n==r->min)
+ if (n==min)
     {
      Set_Min_Mask(propag)
      Set_Min_Max_Mask(propag)
@@ -873,7 +874,7 @@ start:
      goto do_propag;
     }
 
- if (n==r->max)
+ if (n==max)
     {
      Set_Max_Mask(propag)
      Set_Min_Max_Mask(propag)
