@@ -6,7 +6,7 @@
 /* Descr.: FD Range Implementation                                         */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,6 +22,7 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 #include <stdio.h>
+#include <string.h>
 
 #include "bool.h"
 
@@ -282,7 +283,7 @@ int Vector_Next_After(Vector vec,int n)
 
  if (n>=0)                                             /* n >= 0 find next */
     {
-     if (n>vec_max_integer)
+     if (n > vec_max_integer)
          return -1;
 
      word_no=Word_No(n);
@@ -332,7 +333,7 @@ int Vector_Next_Before(Vector vec,int n)
  VecWord word;
  int     bit;
 
- if (n<=vec_max_integer)             /* n <= vec_max_integer find previous */
+ if (n <= vec_max_integer)           /* n <= vec_max_integer find previous */
     {
      if (n<0)
          return -1;
@@ -512,14 +513,14 @@ void Vector_Add_Vector(Vector vec,Vector vec1)
 
          x=vec_elem+vec_elem1;
 
-         if (x>vec_max_integer)
+         if (x > vec_max_integer)
              goto loop1;
 
          Vector_Set_Value(vec,x);
 
      VECTOR_END_ENUM
 
-loop1:
+loop1: ;
 
  VECTOR_END_ENUM
 }
@@ -554,7 +555,7 @@ void Vector_Sub_Vector(Vector vec,Vector vec1)
          Vector_Set_Value(vec,x);
      VECTOR_END_ENUM
 
-loop1:
+loop1: ;
 
  VECTOR_END_ENUM
 }
@@ -583,14 +584,14 @@ void Vector_Mul_Vector(Vector vec,Vector vec1)
 
          x=vec_elem*vec_elem1;
 
-         if (x>vec_max_integer)
+         if (x > vec_max_integer)
              goto loop1;	   
 
          Vector_Set_Value(vec,x);
 
      VECTOR_END_ENUM
 
-loop1:
+loop1: ;
 
  VECTOR_END_ENUM
 }
@@ -762,7 +763,7 @@ void Vector_Mul_Value(Vector vec,int n)
 
      x=vec_elem*n;
 
-     if ((unsigned) x>vec_max_integer)
+     if ((unsigned) x > (unsigned) vec_max_integer)
          return;
 
      Vector_Set_Value(vec,x);
@@ -826,7 +827,7 @@ void Vector_Mod_Value(Vector vec,int n)
  VECTOR_BEGIN_ENUM(aux_vec,vec_elem)
 
      x=vec_elem%n;
-     if ((unsigned) x<=vec_max_integer)
+     if ((unsigned) x <= (unsigned) vec_max_integer)
          Vector_Set_Value(vec,x);
 
  VECTOR_END_ENUM
@@ -1119,7 +1120,7 @@ void Range_Set_Value(Range *range,int n)
         }
 
      Range_Becomes_Sparse(range);
-     if ((unsigned) n <= vec_max_integer)
+     if ((unsigned) n <= (unsigned) vec_max_integer)
         {
          Vector_Set_Value(range->vec,n);
          Range_From_Vector(range);
@@ -1130,7 +1131,7 @@ void Range_Set_Value(Range *range,int n)
      return;
     }
 
- if ((unsigned) n > vec_max_integer)
+ if ((unsigned) n > (unsigned) vec_max_integer)
     {
      range->extra_cstr=TRUE;
      return;
@@ -1175,14 +1176,14 @@ void Range_Reset_Value(Range *range,int n)
         }
 
      Range_Becomes_Sparse(range);
-     if ((unsigned) n <= vec_max_integer)
+     if ((unsigned) n <= (unsigned) vec_max_integer)
          Vector_Reset_Value(range->vec,n);
 
      return;
     }
 
 
- if ((unsigned) n > vec_max_integer)
+ if ((unsigned) n > (unsigned) vec_max_integer)
      return;
 
 
