@@ -6,7 +6,7 @@
 /* Descr.: FD to C macros - header file                                    */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -292,8 +292,8 @@
      if (Fd_Variable_Is_Ground(fdv_adr))                                    \
         {                                                                   \
          int n=Min(fdv_adr);                                                \
-         if (n<(t_min) || n>(t_max))    /* detects also if the initial   */ \
-            {                                                               \
+         if (n<(int) (t_min) || n>(int) (t_max))   /* detects also if the */\
+            {                                   /* initial range is empty */\
              ret_val=FALSE;                                                 \
              goto lab_exit;                                                 \
             }                                                               \
