@@ -6,7 +6,7 @@
 /* Descr.: atom table management - header file                             */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,6 +22,13 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 
+#if 1
+#define OPTIM_1_CHAR_ATOM
+#endif
+
+
+
+
 /*---------------------------------*/
 /* Constants                       */
 /*---------------------------------*/
@@ -109,9 +116,9 @@ typedef struct                           /* Atom information               */
 
        int     atom_end_of_file;
 
-       int     atom_host_name;
-
+#ifndef OPTIM_1_CHAR_ATOM
        int     atom_char[256];
+#endif
 
     /* int     char_type[256];                    see definition in atom.c */
        char    char_conv[256];
@@ -132,10 +139,10 @@ extern int     atom_false;
 extern int     atom_true;
 
 extern int     atom_end_of_file;
-extern int     atom_host_name;
-
 
+#ifndef OPTIM_1_CHAR_ATOM
 extern int     atom_char[];
+#endif
 
 extern char    char_conv[];
 extern int     char_type[];
@@ -160,3 +167,14 @@ int       Find_Atom             (char *name);
 int       Gen_New_Atom          (char *prefix,int hash);
 int       Find_Next_Atom        (int last_atom);
 
+
+
+#ifdef OPTIM_1_CHAR_ATOM
+
+#define ATOM_CHAR(c)            ((int) (unsigned char) (c))
+
+#else
+
+#define ATOM_CHAR(c)            (atom_char[(int) (unsigned char) (c)])
+
+#endif
