@@ -1,12 +1,12 @@
 /*-------------------------------------------------------------------------*/
 /* GNU Prolog                                                              */
 /*                                                                         */
-/* Part  : Prolog engine                                                   */
+/* Part  : configuration                                                   */
 /* File  : bool.h                                                          */
 /* Descr.: boolean type definition - header file                           */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,8 +22,8 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 
-#ifndef TYPE_BOOL
-#define TYPE_BOOL
+#ifndef _BOOL_H
+#define _BOOL_H
 
 #ifdef FALSE
 
@@ -51,4 +51,4 @@
 typedef int Bool;
 #endif
 
-#endif
+#endif /* !_BOOL_H */
