@@ -6,7 +6,7 @@
 /* Descr.: general engine - header file                                    */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -104,9 +104,6 @@ Bool      Try_Execute_Top_Level (void);
 
 void      Switch_Reg_Bank       (WamWord *new_reg_bank);
 
-void      Save_All_Regs         (WamWord *buff_all_regs);
-void      Restore_All_Regs      (WamWord *buff_all_regs);
-
 Bool      Call_Prolog           (CodePtr codep);
 Bool      Call_Prolog_Next_Sol  (WamWord *query_b);
 void      Keep_Rest_For_Prolog  (WamWord *query_b);
@@ -118,4 +115,3 @@ void      Execute_A_Continuation(CodePtr codep);
 
 #define   Goto_Predicate(p,n)   ((*Prolog_Predicate(p,n))())
 
-#define   DBGPRINTF             printf
