@@ -6,7 +6,7 @@
 /* Descr.: general engine                                                  */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -49,12 +49,23 @@ void Call_Compiled(CodePtr codep)
 
 #ifndef MAP_REG_BANK
 
-#if defined(M_ix86_linux) || defined(M_ix86_cygwin)
+#if defined(M_ix86_linux) || defined(M_ix86_cygwin) || \
+    defined(M_ix86_sco) || defined(M_ix86_bsd)
+
  register WamWord *rb asm("%ebx")=reg_bank;
+
+#elif defined(M_ix86_win32)
+
+ _asm mov ebx,reg_bank
+
 #elif defined(M_powerpc_linux)
+
  register WamWord *rb asm("15")=reg_bank;
+
 #elif defined(M_sparc)
+
  register WamWord *rb asm("%l0")=reg_bank;
+
 #endif
 
 #endif
