@@ -6,7 +6,7 @@
 /* Descr.: general header file                                             */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,9 +22,8 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 
-#include "sys_conf.h"
+#include "gp_config.h"
 #include "pl_params.h"
-#include "mem_alloc.h"
 #include "hash.h"
 #include "bool.h"
 #include "engine.h"
@@ -32,7 +31,9 @@
 #include "pred.h"
 #include "misc.h"
 #include "oper.h"
+#include "wam_regs.h"
 #include "wam_archi.h"
+#include "machine1.h"
 #include "machine.h"
 #include "obj_chain.h"
 #include "wam_inst.h"
