@@ -3,10 +3,10 @@
 /*                                                                         */
 /* Part  : Prolog engine                                                   */
 /* File  : machine.c                                                       */
-/* Descr.: machine Dependent Features                                      */
+/* Descr.: machine dependent features                                      */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -26,21 +26,33 @@
 #include <signal.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <pwd.h>
 #include <ctype.h>
 #include <string.h>
-#include <unistd.h>
+#include <time.h>
 #include <sys/types.h>
-#include <sys/param.h>
 #include <sys/stat.h>
 
-#include "engine_pl.h"
+#include "gp_config.h"
+
+#ifdef M_ix86_win32
+#include <windows.h>
+#include <process.h>
+#include <direct.h>
+#else
+#include <pwd.h>
+#include <unistd.h>
+#include <sys/param.h>
+#endif
 
 #ifdef M_ix86_cygwin
+#include <process.h>
 #include <sys/cygwin.h>
 #include <sys/times.h>
 #endif
 
+#include "engine_pl.h"         /* before netdb.h which declares a function */
+                         /* gcc cannot define a global reg var after a fct */
+
 #ifndef NO_USE_SOCKETS
 #include <netdb.h>
 #include <netinet/in.h>
@@ -52,10 +64,14 @@
 
 
 
+
 /*---------------------------------*/
 /* Constants                       */
 /*---------------------------------*/
 
+#define M_SECURITY_MARGIN          128                      /* in WamWords */
+#define M_MAGIC                    0x12345678
+
 #define UNKOWN_SYS_ERRNO           "Unknown error (%d)"
 
 
@@ -67,8 +83,8 @@
 
 #define ERR_CANNOT_OPEN_DEV0       "Cannot open /dev/zero : %s"
 #define ERR_CANNOT_UNMAP           "unmap failed : %s"
-#define ERR_CANNOT_SHMGET          "shmget failed : %s"
-#define ERR_CANNOT_SHMAT           "shmat failed : %s"
+
+#define ERR_CANNOT_PROTECT         "VirtualProtect failed : %lu"
 
 #define ERR_CANNOT_EXEC_GETCWD     "cannot execute getcwd"
 
@@ -127,8 +143,10 @@ void      Fill_Magic_Adr_Table  (void);
 
 #endif
 
+#ifndef M_ix86_win32
 static
 void      SIGSEGV_Handler       ();
+#endif
 
 static 
 void      Stack_Overflow        (int stk_nb);
@@ -158,11 +176,16 @@ void Init_Machine(void)
  tps=sysconf(_SC_CLK_TCK);
  rt0=times(&time_buf0);
 #endif
+
+ tzset();
+
  start_user_time  =M_User_Time();
  start_system_time=M_System_Time();
  start_real_time  =M_Real_Time();
 
  getcwd(cur_work_dir,sizeof(cur_work_dir)-1);
+
+ Init_Machine1();
 }
 
 
@@ -206,7 +229,9 @@ void M_Allocate_Stacks(void)
 
 #ifdef M_USE_MMAP
 
+#ifndef M_ix86_win32
 #include <sys/mman.h>
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* M_ALLOCATE_STACKS                                                       */
@@ -215,13 +240,11 @@ void M_Allocate_Stacks(void)
 void M_Allocate_Stacks(void)
 
 {
- int      fd;
  int      len=0;
  WamWord *addr;
  int      i;
-
- page_size=getpagesize()/sizeof(WamWord);
-#ifndef M_alpha_osf
+#if !defined(M_alpha_osf) && !defined(M_ix86_win32)
+ int      fd;
 
  fd=open("/dev/zero",0);
 
@@ -230,6 +253,8 @@ void M_Allocate_Stacks(void)
 
 #endif
 
+ page_size=getpagesize()/sizeof(WamWord);
+
  for(i=0;i<NB_OF_STACKS;i++)
     {
      stk_tbl[i].size=Round_Up(stk_tbl[i].size,page_size);
@@ -240,6 +265,10 @@ void M_Allocate_Stacks(void)
  addr=(WamWord *) Round_Down((long) addr,getpagesize());
  addr-=len;
 
+#ifdef M_ix86_win32
+ addr=(WamWord *) VirtualAlloc(addr,len*sizeof(WamWord),
+                               MEM_RESERVE|MEM_COMMIT,PAGE_READWRITE);
+#else
  addr=(WamWord *) mmap((caddr_t) addr,len*sizeof(WamWord),
                        PROT_READ|PROT_WRITE,
 #ifndef M_alpha_osf
@@ -248,20 +277,35 @@ void M_Allocate_Stacks(void)
                        MAP_ANON|MAP_PRIVATE|MAP_FIXED,-1,
 #endif
                        0);
+#endif
+
 
 #ifdef DEBUG
  DBGPRINTF("start addr:%lx   size:%ld\n",addr,len*sizeof(WamWord));
 #endif
 
+#ifdef M_ix86_win32
+ if (addr==NULL)
+#else
  if ((long) addr<0)
+#endif
      Fatal_Error(ERR_STACKS_ALLOCATION);
 
  for(i=0;i<NB_OF_STACKS;i++)
     {
      stk_tbl[i].stack=addr;
      addr+=stk_tbl[i].size;
+#ifdef M_ix86_win32
+    {
+     DWORD old_prot;
+
+     if (!VirtualProtect(addr,page_size,PAGE_NOACCESS,&old_prot))
+         Fatal_Error(ERR_CANNOT_PROTECT,GetLastError());
+    }
+#else
      if (munmap((caddr_t) addr,page_size)== -1)
          Fatal_Error(ERR_CANNOT_UNMAP,M_Sys_Err_String(errno));
+#endif
 
      addr+=page_size;
     }
@@ -270,7 +314,8 @@ void M_Allocate_Stacks(void)
  Fill_Magic_Adr_Table();
 #endif
 
-#if defined(M_sparc_solaris)
+#if defined(M_sparc_solaris) || defined(M_ix86_solaris) || \
+    defined(M_ix86_sco)
 {
  struct sigaction act;
 
@@ -282,12 +327,12 @@ void M_Allocate_Stacks(void)
  sigaction(SIGSEGV,&act,NULL);
 }
 
-#else
+#elif !defined(M_ix86_win32)
  signal(SIGSEGV,(void (*)()) SIGSEGV_Handler);
 #endif
 }
 
-#endif
+#endif /* M_USE_MMAP */
 
 
 
@@ -316,6 +361,47 @@ static void Fill_Magic_Adr_Table(void)
 
 
 
+#ifdef M_ix86_win32
+
+/*-------------------------------------------------------------------------*/
+/* GETPAGESIZE                                                             */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+int getpagesize(void)
+
+{
+ SYSTEM_INFO si;
+
+ GetSystemInfo(&si);
+ return si.dwPageSize;     
+}
+
+
+
+
+static long *fault_addr;
+
+/*-------------------------------------------------------------------------*/
+/* IS_WIN32_SEGV                                                           */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+int Is_Win32_SEGV(void *exp)
+
+{
+ LPEXCEPTION_POINTERS err=(LPEXCEPTION_POINTERS) exp;
+ PEXCEPTION_RECORD per=err->ExceptionRecord;
+
+ if (per->ExceptionCode!=EXCEPTION_ACCESS_VIOLATION)
+     return EXCEPTION_CONTINUE_SEARCH;
+
+ fault_addr=(long *) (per->ExceptionInformation[1]);
+ return EXCEPTION_EXECUTE_HANDLER;
+}    
+
+#endif /* M_ix86_win32 */
+
+
+
 
 /*-------------------------------------------------------------------------*/
 /* SIGSEGV_HANDLER                                                         */
@@ -325,7 +411,7 @@ static void Fill_Magic_Adr_Table(void)
 
 static void SIGSEGV_Handler(int sig,int code,int scp,WamWord *addr)
 
-#elif defined(M_sparc_solaris)
+#elif defined(M_sparc_solaris) || defined(M_ix86_solaris)
 
 void SIGSEGV_Handler(int sig,siginfo_t *sip)
 
@@ -342,6 +428,21 @@ static void SIGSEGV_Handler(int sig,struct sigcontext_struct scp)
 static void SIGSEGV_Handler(int sig,struct sigcontext scp)
 #endif
 
+#elif defined(M_ix86_sco)
+#define _XOPEN_SOURCE_EXTENDED
+#include <signal.h>
+#include <sys/siginfo.h>
+
+static void SIGSEGV_Handler(int sig,siginfo_t *si)
+
+#elif defined(M_ix86_bsd)
+
+static void SIGSEGV_Handler(int sig,int code, struct sigcontext *scp)
+
+#elif defined(M_ix86_win32)
+
+void SIGSEGV_Handler(void)
+
 #else
 
 static void SIGSEGV_Handler(int sig)
@@ -359,7 +460,7 @@ static void SIGSEGV_Handler(int sig)
 
  WamWord *addr=(WamWord *) (scp->sc_traparg_a0);
 
-#elif defined(M_sparc_solaris)
+#elif defined(M_sparc_solaris) || defined(M_ix86_solaris)
 
  WamWord *addr=(WamWord *) sip->si_addr;
 
@@ -367,10 +468,22 @@ static void SIGSEGV_Handler(int sig)
 
  WamWord *addr=(WamWord *) scp.cr2;
 
+#elif defined(M_ix86_sco)
+
+ WamWord *addr=(WamWord *) si->si_addr;
+
+#elif defined(M_ix86_bsd)
+
+ WamWord *addr=(WamWord *) scp->sc_err;
+
 #elif defined(M_powerpc_linux)
 
  WamWord *addr=(WamWord *) scp.regs->dar;
 
+#elif defined(M_ix86_win32)
+
+ WamWord *addr=(WamWord *) fault_addr;
+
 #endif
 
  int i;
@@ -477,111 +590,6 @@ char *M_Sys_Err_String(int err_no)
 
 
 
-/*-------------------------------------------------------------------------*/
-/* M_SHELL                                                                 */
-/*                                                                         */
-/* Invoke a shell (eventually passing a cmd if != NULL)                    */
-/*-------------------------------------------------------------------------*/
-int M_Shell(char *cmd)
-
-{
- char *env_var_name;
- char *default_path;
- char *before;
- char *option;
- char *after;
- char *shell_path;
- char  buff[1024];
-
-#if defined(__unix__) || defined(__CYGWIN__)
-
- env_var_name="SHELL";
- default_path="/bin/sh";
- option="-c '";
- before="exec ";
- after="'";
-
-#elif __msdos__
-
- env_var_name="COMSPEC";
- default_path="command.com";
- option="/c ";
- before="";
- after="";
-
-#else
-
- Fatal_Error("Warning: Shell access not available")
- return -1;
-
-#endif
-
- shell_path=getenv(env_var_name);
- if (shell_path==NULL)
-     shell_path=default_path;
-
- if (cmd)
-     sprintf(buff,"%s%s %s%s%s",before,shell_path,option,cmd,after);
-  else
-     sprintf(buff,"%s%s",before,shell_path);
-
-#ifdef DEBUG
- DBGPRINTF("\n<%s>\n",buff);
-#endif
-
- return system(buff);
-}
-
-
-
-
-#if defined(__unix__) || defined(__CYGWIN__)
-
-#include <sys/types.h>
-#include <sys/wait.h>
-
-#endif
-
-/*-------------------------------------------------------------------------*/
-/* M_SPAWN                                                                 */
-/*                                                                         */
-/* Execute a command with arguments in arg[], (arg[0]=the name of the cmd, */
-/* a NULL must follow the last argument.                                   */
-/* return the status or -1 if cannot execute (errno is not set) or -2 else */
-/* (errno is set).                                                         */
-/*-------------------------------------------------------------------------*/
-int M_Spawn(char *cmd,char *arg[])
-
-{
- int pid,status;
-
- pid=vfork();
-
- if (pid== -1)
-     return -2;
-
- if (pid==0)                                              /* child process */
-    {
-     execvp(cmd,arg);                             /* only returns on error */
-     exit(127);
-    }
-
- if (waitpid(pid,&status,0)<0)
-     return -2;
-
- if (WIFEXITED(status))
-    {
-     status=WEXITSTATUS(status);
-     if (status==127)
-         status= -1;
-    }
-
- return status;
-}
-
-
-
-
 #if defined(__unix__) || defined(__CYGWIN__)
 
 #include <sys/time.h>
@@ -614,9 +622,13 @@ long M_User_Time(void)
  times(&time_buf1);
  user_time=(time_buf1.tms_utime - time_buf0.tms_utime)*1000/tps;
 
+#elif defined(M_ix86_win32)
+
+ user_time=(long) ((double) clock()*1000/CLOCKS_PER_SEC);
+
 #else
 
- Fatal_Error("Warning: user time not available");
+ Fatal_Error("user time not available");
  return 0;
 
 #endif
@@ -652,9 +664,13 @@ long M_System_Time(void)
  times(&time_buf1);
  system_time=(time_buf1.tms_stime - time_buf0.tms_stime)*1000/tps;
 
+#elif defined(M_ix86_win32)
+
+ system_time=0;
+
 #else
 
- Fatal_Error("Warning: system time not available");
+ Fatal_Error("system time not available");
  return 0;
 
 #endif
@@ -682,9 +698,13 @@ long M_Real_Time(void)
   real_time=(tv.tv_sec *1000) +
             (tv.tv_usec/1000);
 
+#elif defined(M_ix86_win32)
+
+ real_time=M_User_Time();
+
 #else
 
- Fatal_Error("Warning: real time not available");
+ Fatal_Error("real time not available");
  return 0;
 
 #endif
@@ -780,6 +800,9 @@ char *M_Host_Name_From_Name(char *host_name)
 
 {
  static char     buff[4096];
+#ifdef M_ix86_win32
+        long     length=sizeof(buff);
+#endif
 #ifdef INET_MANAGEMENT
  struct hostent *host_entry;
 #endif
@@ -787,7 +810,11 @@ char *M_Host_Name_From_Name(char *host_name)
  if (host_name==NULL)
     {
      host_name=buff;
+#if defined(M_ix86_win32) && defined(NO_USE_SOCKETS)
+     if (GetComputerName(buff,&length)==0)
+#else
      if (gethostname(buff,sizeof(buff)))
+#endif
         {
          strcpy(buff,"unknown host name");
          goto finish;
@@ -825,7 +852,8 @@ char *M_Host_Name_From_Adr(char *host_address)
  struct hostent *host_entry;
  struct in_addr  iadr;
 
-#if defined(M_sparc_sunos) || defined(M_sparc_solaris) || defined(M_ix86_cygwin)
+#if defined(M_sparc_sunos) || defined(M_sparc_solaris) || \
+    defined(M_ix86_cygwin) || defined(M_ix86_solaris)
  if ((iadr.s_addr=inet_addr(host_address))== -1)
 #else
  if (inet_aton(host_address,&iadr)==0)
@@ -858,7 +886,7 @@ static char *Host_Name_From_Alias(struct hostent *host_entry)
  char **alias;
  char  *p;
 
- name=host_entry->h_name;
+ name=(char *) host_entry->h_name;
  alias=host_entry->h_aliases;
  while((p=strchr(name,'.'))==NULL && *alias)
      name=*alias++;
@@ -866,10 +894,9 @@ static char *Host_Name_From_Alias(struct hostent *host_entry)
  if (p)
      return name;
 
- return host_entry->h_name;
+ return (char *) host_entry->h_name;
 }
 
-
 #endif
 
 
@@ -922,17 +949,30 @@ char *M_Absolute_Path_Name(char *src)
  int     res=0;
  char   *dst;
  char   *p,*q;
- struct 
- passwd *pw;
+#ifdef M_ix86_win32
+ char    c;
+#endif
+
 
  dst=buff[res];
  while((*dst++=*src))                                   /* expand $VARNAME */
     {
-     if (*src++=='$')
+#ifdef M_ix86_win32
+     c=*src;
+#endif
+     if (*src++=='$'
+#ifdef M_ix86_win32
+         || c=='%'
+#endif
+        )
         {
          p=dst;
          while(isalnum(*src))
              *dst++=*src++;
+#ifdef M_ix86_win32
+         if (c=='%' && *src!='%')
+             continue;
+#endif
          *dst='\0';
          q=getenv(p);
          if (q)
@@ -940,12 +980,21 @@ char *M_Absolute_Path_Name(char *src)
              p--;
              strcpy(p,q);
              dst=p+strlen(p);
+#ifdef M_ix86_win32
+             if (c=='%')
+                 src++;
+#endif
             }
+#ifdef M_ix86_win32
+          else
+             if (c=='%')
+                 *dst++=*src++;
+#endif
         }
     }
  *dst='\0';
 
-#if defined(__msdos__) || defined(M_ix86_cygwin)
+#if defined(M_ix86_win32) || defined(M_ix86_cygwin)
  for(src=buff[res];*src;src++)                              /* \ becomes / */
      if (*src=='\\')
          *src='/';
@@ -957,7 +1006,7 @@ char *M_Absolute_Path_Name(char *src)
 
  if (buff[res][0]=='~')
     {
-     if (buff[res][1]=='/' || buff[res][1]=='\0')      /* ~/... read $HOME */
+     if (buff[res][1]==DIR_SEP_C || buff[res][1]=='\0')  /* ~/... cf $HOME */
         {
          if ((p=getenv("HOME"))==NULL)
              return NULL;
@@ -968,8 +1017,10 @@ char *M_Absolute_Path_Name(char *src)
 #if defined(__unix__) || defined(__CYGWIN__)
       else                                        /* ~user/... read passwd */
         {
+         struct passwd *pw;
+
          p=buff[res]+1;
-         while(*p && *p!='/')
+         while(*p && *p!=DIR_SEP_C)
               p++;
 
          buff[res][0]=*p;
@@ -990,11 +1041,17 @@ char *M_Absolute_Path_Name(char *src)
  res=1-res;
 #endif
 
- if (buff[res][0]!='/')                                /* add cur_work_dir */
+#ifdef M_ix86_win32
+ if (_fullpath(buff[1-res],buff[res],MAXPATHLEN)==NULL)
+     return NULL;
+ res=1-res;
+#else
+ if (buff[res][0]!=DIR_SEP_C)                          /* add cur_work_dir */
     {
      sprintf(buff[1-res],"%s/%s",cur_work_dir,buff[res]);
      res=1-res;
     }
+#endif
 
  res=1-res;
  src=buff[1-res];
@@ -1002,29 +1059,29 @@ char *M_Absolute_Path_Name(char *src)
 
  while((*dst++=*src))
     {
-     if (*src++!='/')
+     if (*src++!=DIR_SEP_C)
          continue;
 
 collapse:
-#ifndef M_ix86_cygwin
-     while(*src=='/')                            /* collapse /////... as / */
+#ifndef M_ix86_cygwin  /* CYGWIN uses //<drive>/<path> for <drive>:\<path> */
+     while(*src==DIR_SEP_C)                      /* collapse /////... as / */
          src++;
 #endif
      if (*src!='.')
          continue;
 
-     if (src[1]=='/' || src[1]=='\0')                       /* /./ removed */
+     if (src[1]==DIR_SEP_C || src[1]=='\0')                 /* /./ removed */
         {
          src++;
          goto collapse;
         }
 
-      if (src[1]!='.' || (src[2]!='/' && src[2]!='\0'))
+      if (src[1]!='.' || (src[2]!=DIR_SEP_C && src[2]!='\0'))
           continue;
                                                               /* case /../ */
       src+=2;
       p=dst-2;
-      while(p>=buff[res] && *p!='/')
+      while(p>=buff[res] && *p!=DIR_SEP_C)
           p--;
 
       if (p<buff[res])
@@ -1033,7 +1090,7 @@ collapse:
       dst=p;
     }
 
- if (dst[-2]=='/')
+ if (dst[-2]==DIR_SEP_C)
      dst[-2]='\0';                                        /* remove last / */
 
  return buff[res];
