@@ -6,7 +6,7 @@
 /* Descr.: C Compiler options and WAM Configuration                        */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -27,7 +27,8 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "config.h"
+#include "gp_config.h"
+#include "machine.h"
 
 
 
@@ -38,7 +39,7 @@
 
 #define STR_LENGTH                 512
 
-#define FILE_SYS_CONF_H            "sys_conf.h"
+#define FILE_WAM_REGS_H            "wam_regs.h"
 
 #define FILE_WAM_ARCHI_DEF         "wam_archi.def"
 #define FILE_WAM_ARCHI_H           "wam_archi.h"
@@ -92,7 +93,7 @@ typedef struct
 /*---------------------------------*/
 
 char  save_str[STR_LENGTH];
-FILE *fsys;
+FILE *fw_r;
 
 
 
@@ -131,21 +132,21 @@ int main(void)
  if (*M_CPU=='?')
     {
      printf("*** This architecture is not supported ***\n");
-     exit(-1);
+     return 1;
     }
 
 
  if (WORD_SIZE!=(sizeof(long)*8))
     {
      printf("Bad Value of WORD_SIZE - should be %d\n",(sizeof(long)*8));
-     exit(1);
+     return 1;
     }
 
- if ((fsys=fopen(FILE_SYS_CONF_H,"wt"))==NULL)
-     Fatal_Error("cannot open %s",FILE_SYS_CONF_H);
+ if ((fw_r=fopen(FILE_WAM_REGS_H,"wb"))==NULL)
+     Fatal_Error("cannot open %s",FILE_WAM_REGS_H);
 
- fprintf(fsys,"/* this file is automatically generated by pl_config.c*/\n");
- fprintf(fsys,"\n#include \"config.h\"\n\n");
+ fprintf(fw_r,"/* this file is automatically generated by pl_config.c */\n");
+ fprintf(fw_r,"\n#include \"gp_config.h\"\n\n");
 
  printf("\n");
  printf("\t-------------------------------\n");
@@ -167,6 +168,15 @@ int main(void)
  "No"
 #endif
  );
+#ifdef M_ix86_win32
+ printf("Use GUI console   : %s\n",
+#ifdef W32_GUI_CONSOLE
+ "Yes"
+#else
+ "No"
+#endif
+ );
+#endif
  printf("Use sockets       : %s\n",
 #ifndef NO_USE_SOCKETS
  "Yes"
@@ -192,7 +202,7 @@ int main(void)
  nb_of_used_regs=Generate_Archi();
  Write_C_Compiler_Info(nb_of_used_regs);
 
- fclose(fsys);
+ fclose(fw_r);
 
 #ifndef NO_USE_REGS
  printf("Used register(s)  : ");
@@ -205,7 +215,7 @@ int main(void)
  printf("\n");
  printf("\t------------------------------\n\n");
 
- exit(0);
+ return 0;
 }
 
 
@@ -221,15 +231,15 @@ void Write_C_Compiler_Info(int nb_of_used_regs)
  char  *used_regs[]=M_USED_REGS;
  int    i;
 
- fputc('\n',fsys);
- fprintf(fsys,"#define CFLAGS_REGS\t\t\"");
+ fputc('\n',fw_r);
+ fprintf(fw_r,"#define CFLAGS_REGS\t\t\"");
  for(i=0;i<nb_of_used_regs;i++)
     {
-     fprintf(fsys,CFLAGS_PREFIX_REG,used_regs[i]);
-     fputc(' ',fsys);
+     fprintf(fw_r,CFLAGS_PREFIX_REG,used_regs[i]);
+     fputc(' ',fw_r);
     }
 
- fputs("\"\n",fsys);
+ fputs("\"\n",fw_r);
 }
 
 
@@ -250,12 +260,9 @@ int Generate_Archi(void)
  if ((f=fopen(FILE_WAM_ARCHI_DEF,"rt"))==NULL)
      Fatal_Error("cannot open %s",FILE_WAM_ARCHI_DEF);
 
- if ((g=fopen(FILE_WAM_ARCHI_H,"wt"))==NULL)
+ if ((g=fopen(FILE_WAM_ARCHI_H,"wb"))==NULL)
      Fatal_Error("cannot open %s",FILE_WAM_ARCHI_H);
 
- if ((f=fopen(FILE_WAM_ARCHI_DEF,"rt"))==NULL)
-     Fatal_Error("cannot open %s",FILE_WAM_ARCHI_DEF);
-
  while(!feof(f) && fgets(str,sizeof(str),f))
     {
      if (*str!='@')
@@ -457,7 +464,7 @@ int Generate_Regs(FILE *f,FILE *g)
     {
      nb_of_used_regs++;
      fprintf(g,"register WamWord \t\t*reg_bank asm (\"%s\");\n\n",*p);
-     fprintf(fsys,"#define MAP_REG_BANK\t\t\"%s\"\n\n",*p++);
+     fprintf(fw_r,"#define MAP_REG_BANK\t\t\"%s\"\n\n",*p++);
     }
   else
     {
@@ -477,7 +484,7 @@ int Generate_Regs(FILE *f,FILE *g)
              nb_of_used_regs++;
              fprintf(g,"register %s\t\t%-3s asm (\"%s\");\n",
                      dp->type,dp->name,*p);
-             fprintf(fsys,"#define MAP_REG_%-10s\t\"%s\"\n",dp->name,*p++);
+             fprintf(fw_r,"#define MAP_REG_%-10s\t\"%s\"\n",dp->name,*p++);
              if (!*p)
                  fprintf(g,"\n\n");
             }
@@ -485,7 +492,7 @@ int Generate_Regs(FILE *f,FILE *g)
             {
              fprintf(g,"#define %s\t\t\t((%s)\t (reg_bank[%s%d]))\n",
                      dp->name,dp->type,str_base,nb_not_alloc);
-             fprintf(fsys,"#define MAP_OFFSET_%-6s\t((%s%d)*%d)\n",
+             fprintf(fw_r,"#define MAP_OFFSET_%-6s\t((%s%d)*%d)\n",
                      dp->name,str_base,nb_not_alloc++,(int) sizeof(long));
             }
         }
@@ -531,6 +538,39 @@ int Generate_Regs(FILE *f,FILE *g)
 
  
 
+ fprintf(g,"\n\n\n\n#define Save_All_Regs(buff_save)\t\t\\\n");
+ fprintf(g,"    {               \t\t\t\t\\\n");
+
+ k=0;
+ for(i=0;i<10;i++)
+     for(j=0;j<nb_reg[i];j++)
+        {
+         dp= &reg[i][j];
+         fprintf(g,"     buff_save[%d]=(WamWord) %-6s;\t\t\\\n",k,dp->name);
+         k++;
+        }
+
+ fprintf(g,"    }\n");
+
+
+
+ fprintf(g,"\n\n\n\n#define Restore_All_Regs(buff_save)\t\t\\\n");
+ fprintf(g,"    {               \t\t\t\t\\\n");
+
+ k=0;
+ for(i=0;i<10;i++)
+     for(j=0;j<nb_reg[i];j++)
+        {
+         dp= &reg[i][j];
+         fprintf(g,"     %-6s=(%s)\tbuff_save[%d];\t\t\\\n",
+                 dp->name,dp->type,k);
+         k++;
+        }
+
+ fprintf(g,"    }\n");
+
+
+
  fprintf(g,"\n\n\n\n#define NB_OF_USED_MACHINE_REGS %d\n",nb_of_used_regs);
  fprintf(g,"\n\n\n\n#define Save_Machine_Regs(buff_save)\t\t\\\n");
  fprintf(g,"    {               \t\t\t\t\\\n");
