@@ -6,7 +6,7 @@
 /* Descr.: WAM instruction implementation                                  */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -731,9 +731,9 @@ void Allocate(int n)
  adr=E;
  E=Local_Top+ENVIR_STATIC_SIZE+n;
 
- CPE(E) =(WamWord) CP;
+ CPE(E) =(WamCont) CP;
  BCIE(E)=BCI;
- EE(E)  =(WamWord) adr;
+ EE(E)  =(WamWord *) adr;
 }
 
 
