@@ -6,7 +6,7 @@
 /* Descr.: line editor                                                     */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -23,20 +23,24 @@
 /*-------------------------------------------------------------------------*/
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include <string.h>
 #include <signal.h>
-#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/time.h>
-#include <sys/types.h>
-
 
 #include "stty.h"
 #include "char_io.h"
 #include "linedit.h"
 
+#if defined(__unix__) || defined(__CYGWIN__)
+#include <unistd.h>
+#include <sys/time.h>
+#elif defined(_MSC_VER)
+#include <time.h>
+#endif
+
 
 
 
@@ -44,7 +48,7 @@
 /* Constants                       */
 /*---------------------------------*/
 
-#define LINEDIT_VERSION            "2.0"
+#define LINEDIT_VERSION            "2.1"
 
 #define MAX_HISTORY_LINES          64
 #define MAX_SEPARATORS             256
@@ -98,6 +102,9 @@ struct comp_node
 static char      separators[MAX_SEPARATORS]=DEFAULT_SEPARATORS;
 static int       ins_mode=1;
 
+static char     *global_str;
+static char     *global_pos;
+static char     *global_end;
 
 static char      clipboard[4096]="";
 
@@ -121,6 +128,15 @@ static CompNode *comp_cur_match;
 /* Function Prototypes             */
 /*---------------------------------*/
 
+static
+void      Backd                 (int fd_out,int n);
+static
+void      Forwd                 (int fd_out,int n,char *str);
+static
+void      Displ                 (int fd_out,int n,char *str);
+static
+void      Erase                 (int fd_out,int n);
+
 static
 int       New_Char              (int c,int ins_mode,char *str,int size,
                                  int fd_out,char **p_pos,char **p_end);
@@ -153,11 +169,7 @@ void      Display_Help          (int fd_out);
 
 
 
-
 #define NewLn()  {LE_Put_Char('\n',fd_out);  }
-#define Backd(n) {int i=n; while(i--) LE_Put_Char('\b',fd_out); }
-#define Forwd(n) {int i=n; char *z=pos; while(i--) LE_Put_Char(*z++,fd_out);}
-#define Erase(n) {int i=n; while(i--) LE_Put_Char(' ',fd_out); Backd(n) }
 
 
 #define Hist_Inc(n)   { if (++(n)==MAX_HISTORY_LINES) (n)=0; }
@@ -174,9 +186,12 @@ void      Display_Help          (int fd_out);
 
 #if defined(__unix__) || defined(__CYGWIN__)
 
-#include <unistd.h>
 #define Emit_Signal(sig)      kill(getpid(),sig)
 
+#elif defined(_MSC_VER)
+
+#define Emit_Signal(sig)      raise(sig)
+
 #else
 
 #define Emit_Signal(sig)      printf("\nlinedit: should send a signal(%d) - to do\n",sig)
@@ -186,7 +201,6 @@ void      Display_Help          (int fd_out);
 
 
 
-
 /*-------------------------------------------------------------------------*/
 /* LE_GETS                                                                 */
 /*                                                                         */
@@ -236,11 +250,11 @@ char *LE_FGets(char *str,int size,int fd_in,int fd_out,char *prompt,
 
  size--;                                                    /* -1 for '\0' */
 
-
  LE_Init(fd_in,fd_out);
+ global_str=str;
 
 #if 1                           /* avoid to redisplay a buffered full line */
- while(LE_Input_Is_Not_Empty(fd_in))
+ while(LE_Kbd_Is_Not_Empty(fd_in))
     {
      if (end-str>=size)
         {
@@ -269,6 +283,9 @@ char *LE_FGets(char *str,int size,int fd_in,int fd_out,char *prompt,
 
  for(;;)
     {
+     global_pos=pos;
+     global_end=end;
+
      c=LE_Get_Char(fd_in);
 one_char:
      *end=' ';                              /* to allow for separator test */
@@ -276,28 +293,28 @@ one_char:
      if ((sig=LE_Which_Signal(c))!=0)
         {
          LE_Term(fd_in,fd_out);
-         c=*str;
-         *str='\0';                /* to allow the handler to use/test str */
+         c=*end;
+         *end='\0';                /* to allow the handler to use/test str */
          Emit_Signal(sig);
          LE_Init(fd_in,fd_out);
-         *str=c;
+         *end=c;
 
 re_display_line:                             /* display prompt + full line */
          if (prompt)
              Display_String(fd_out,prompt);
          p=pos;
          pos=str;
-         Forwd(end-str);
+         Displ(fd_out,end-str,pos);
          pos=p;
-         Backd(end-pos);
+         Backd(fd_out,end-pos);
          continue;
         }
 
-     if (c==KEY_EOF)               /* to avoid EOF when typing too much ^D */
+     if (KEY_IS_EOF(c))            /* to avoid EOF when typing too much ^D */
         {
          if (end==str)
             {
-             if (last_was_eof)
+             if (c==KEY_CTRL('D') && last_was_eof)
                  goto error;
               else
                 {
@@ -305,7 +322,7 @@ re_display_line:                             /* display prompt + full line */
                  goto finish;
                 }
             }
-         last_was_eof=1;
+         last_was_eof=(c==KEY_CTRL('D'));
         }
       else
          last_was_eof=0;
@@ -317,14 +334,14 @@ re_display_line:                             /* display prompt + full line */
         {
          case KEY_CTRL('A'):                        /* go to begin of line */
          case KEY_EXT_HOME:
-             Backd(pos-str);
+             Backd(fd_out,pos-str);
              pos=str;
              continue;
 
 
          case KEY_CTRL('E'):                          /* go to end of line */
          case KEY_EXT_END:
-             Forwd(end-pos);
+             Forwd(fd_out,end-pos,pos);
              pos=end;
              continue;
 
@@ -333,7 +350,7 @@ re_display_line:                             /* display prompt + full line */
          case KEY_EXT_LEFT:
              if (pos==str)
                  goto error;
-             Backd(1);
+             Backd(fd_out,1);
              pos--;
              continue;
 
@@ -342,24 +359,24 @@ re_display_line:                             /* display prompt + full line */
          case KEY_EXT_RIGHT:
              if (pos==end)
                  goto error;
-             Forwd(1);
+             Forwd(fd_out,1,pos);
              pos++;
              continue;
 
 
-         case KEY_BASKSPACE:                        /* erase previous char */
+         case KEY_BACKSPACE:                        /* erase previous char */
          case KEY_DELETE:
              if (pos==str)
                  goto error;
 del_last:
              for(p=pos;p<end;p++)
                  p[-1]=*p;
-             Backd(1);
+             Backd(fd_out,1);
              pos--;
              end--;
-             Forwd(end-pos);
-             Erase(1);
-             Backd(end-pos);
+             Displ(fd_out,end-pos,pos);
+             Erase(fd_out,1);
+             Backd(fd_out,end-pos);
              continue;
 
 
@@ -368,7 +385,7 @@ del_last:
              if (pos==end)
                  goto error;
                                     /* simply equivalent to ^F + BACKSPACE */
-             Forwd(1);
+             Forwd(fd_out,1,pos);
              pos++;
              goto del_last;
 
@@ -385,11 +402,11 @@ del_last:
                  p[-n]=*p;
 
              pos=str;
-	     end-=n;
-             Backd(n);
-	     Forwd(end-pos);
-             Erase(n);
-             Backd(end-pos);
+             end-=n;
+             Backd(fd_out,n);
+             Displ(fd_out,end-pos,pos);
+             Erase(fd_out,n);
+             Backd(fd_out,end-pos);
              continue;
 
 
@@ -400,7 +417,7 @@ del_last:
              while(p<end)                 /* add deleted part to clipboard */
                  *q++=*p++;
              *q='\0';
-             Erase(end-pos);
+             Erase(fd_out,end-pos);
              end=pos;
              continue;
 
@@ -445,12 +462,12 @@ del_last:
                  p[-n]=*p;
 
              if (mark<pos)
-                 Backd(n);
+                 Backd(fd_out,n);
              pos=start;
              end-=n;
-             Forwd(end-pos);
-             Erase(n);
-             Backd(end-pos);
+             Displ(fd_out,end-pos,pos);
+             Erase(fd_out,n);
+             Backd(fd_out,end-pos);
              continue;
 
 
@@ -460,7 +477,7 @@ del_last:
              p=Skip(p,str,1,-1);                        /* skip separators */
              p=Skip(p,str,0,-1);                    /* skip non separators */
              p=Skip(p,end,1,+1);                        /* skip separators */
-             Backd(pos-p);
+             Backd(fd_out,pos-p);
              pos=p;
              continue;
 
@@ -470,7 +487,7 @@ del_last:
              p=pos;
              p=Skip(p,end,0,+1);                    /* skip non separators */
              p=Skip(p,end,1,+1);                        /* skip separators */
-             Forwd(p-pos);
+             Forwd(fd_out,p-pos,pos);
              pos=p;
              continue;
 
@@ -481,7 +498,7 @@ del_last:
              if (islower(*p))
                  *p=*p-'a'+'A';
              p=Skip(p,end,0,+1);                    /* skip non separators */
-             Forwd(p-pos);
+             Displ(fd_out,p-pos,pos);
              pos=p;
              continue;
 
@@ -490,9 +507,8 @@ del_last:
              p=pos;
              p=Skip(p,end,1,+1);                        /* skip separators */
              for(;p<end && !Is_A_Separator(*p);p++)
-                 if (isupper(*p))
-                     *p=*p-'A'+'a';
-             Forwd(p-pos);
+                 *p=tolower(*p);
+             Displ(fd_out,p-pos,pos);
              pos=p;
              continue;
 
@@ -501,9 +517,8 @@ del_last:
              p=pos;
              p=Skip(p,end,1,+1);                        /* skip separators */
              for(;p<end && !Is_A_Separator(*p);p++)
-                 if (islower(*p))
-                     *p=*p-'a'+'A';
-             Forwd(p-pos);
+                 *p=toupper(*p);
+             Displ(fd_out,p-pos,pos);
              pos=p;
              continue;
 
@@ -545,18 +560,19 @@ del_last:
          case KEY_CTRL('V'):                /* switch insert mode (on/off) */
          case KEY_EXT_INSERT:
              ins_mode=1-ins_mode;
+             LE_Ins_Mode(ins_mode);
              continue;
 
 
          case KEY_CTRL('T'):                 /* swap last and current char */
              if (pos==str || pos==end)
                  goto error;
-             Backd(1);
+             Backd(fd_out,1);
              pos--;
              w=*pos;
              *pos=pos[1];
              pos[1]=w;
-             Forwd(2);
+             Displ(fd_out,2,pos);
              pos+=2;
              continue;
 
@@ -578,13 +594,13 @@ del_last:
              History_Update_Line(str,end-str,h_no);
              Hist_Dec(h_no);
 write_hist_line:
-             Backd(pos-str);
+             Backd(fd_out,pos-str);
              pos=str;
              p=end;
              end=str+History_Get_Line(str,h_no);
-             Forwd(end-str);
+             Displ(fd_out,end-str,pos);
              if (end<p)
-                 Erase(p-end)
+                 Erase(fd_out,p-end);
              pos=end;
              continue;
 
@@ -614,10 +630,10 @@ try_previous:
 write_hist_match_line:
              p=end;
              end=str+History_Get_Line(str,h_no);
-             Forwd(end-pos);
+             Displ(fd_out,end-pos,pos);
              if (end<p)
-                 Erase(p-end)
-             Backd(end-pos);
+                 Erase(fd_out,p-end);
+             Backd(fd_out,end-pos);
              continue;
 
 
@@ -667,7 +683,7 @@ display_help:
              if (!isprint(c) || c>255)
                 {
                  n=c;
-                 LE_Put_Char('\a',fd_out);
+                 LE_Emit_Beep(fd_out);
                  c=LE_Get_Char(fd_in);
                  if (c!=n)
                      goto one_char;
@@ -677,7 +693,7 @@ display_help:
              if (!New_Char(c,ins_mode,str,size,fd_out,&pos,&end))
                  goto error;
                                              /* brackets ([{ }]): matching */
-             if (LE_Input_Is_Not_Empty(fd_in) || 
+             if (LE_Kbd_Is_Not_Empty(fd_in) || 
                  (q=strchr(close_bracket,c))==NULL)
                  continue;
 
@@ -712,13 +728,24 @@ display_help:
 #endif
                 }
 
-             if (LE_Input_Is_Not_Empty(fd_in))
+             if (LE_Kbd_Is_Not_Empty(fd_in))
                  continue;
 
              n=pos-p;
              q=pos;
-             Backd(n);
-#ifndef NO_USE_SELECT
+             Backd(fd_out,n);
+#ifdef _MSC_VER
+
+            {
+             long t0=clock(),t1;
+
+             do
+                t1=clock();
+             while(!LE_Kbd_Is_Not_Empty(fd_in) && 
+                   (t1-t0)*1000000/CLOCKS_PER_SEC < BRACKET_TIMMING);
+            }
+
+#elif !defined(NO_USE_SELECT)
             {
              fd_set set;
              struct timeval t;
@@ -734,13 +761,13 @@ display_help:
              usleep(BRACKET_TIMMING);
 #endif
              pos=p;
-             Forwd(n);
+             Displ(fd_out,n,pos);
              pos=q;
 bracket_exit:
              continue;
         }
 error:
-     LE_Put_Char('\a',fd_out);
+     LE_Emit_Beep(fd_out);
     }
 
 finish:
@@ -772,7 +799,7 @@ static int New_Char(int c,int ins_mode,char *str,int size,int fd_out,
 
  if (!ins_mode)
     {
-     *pos=c;
+     *pos=(char) c;
      if (++pos>end)
          end=pos;
      LE_Put_Char(c,fd_out);
@@ -782,11 +809,11 @@ static int New_Char(int c,int ins_mode,char *str,int size,int fd_out,
      for(p=end;p>pos;p--)
          *p=p[-1];
 
-     *pos=c;
+     *pos=(char) c;
      end++;
-     Forwd(end-pos);
+     Displ(fd_out,end-pos,pos);
      pos++;
-     Backd(end-pos);
+     Backd(fd_out,end-pos);
     }
 
  *p_pos=pos;
@@ -798,6 +825,121 @@ static int New_Char(int c,int ins_mode,char *str,int size,int fd_out,
 
 
 
+/*-------------------------------------------------------------------------*/
+/* BACKD                                                                   */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static void Backd(int fd_out,int n)
+
+{
+#ifdef W32_GUI_CONSOLE 
+ if (w32gc_backd)
+    {
+     (*w32gc_backd)(n);
+     return;
+    }
+#endif
+ while(n--) 
+     LE_Put_Char('\b',fd_out);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* FORWD                                                                   */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static void Forwd(int fd_out,int n,char *str)
+
+{
+#ifdef W32_GUI_CONSOLE 
+ if (w32gc_forwd)
+    {
+     (*w32gc_forwd)(n);
+     return;
+    }
+#endif
+ while(n--)
+     LE_Put_Char(*str++,fd_out);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* DISPL                                                                   */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static void Displ(int fd_out,int n,char *str)
+
+{
+#ifdef W32_GUI_CONSOLE 
+ if (w32gc_displ)
+    {
+     (*w32gc_displ)(n,str);
+     return;
+    }
+#endif
+ Forwd(fd_out,n,str);
+}
+
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* ERASE                                                                   */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+static void Erase(int fd_out,int n)
+
+{
+ int n0=n;
+
+#ifdef W32_GUI_CONSOLE 
+ if (w32gc_erase)
+    {
+     (*w32gc_erase)(n);
+     return;
+    }
+#endif
+ while(n--)
+     LE_Put_Char(' ',fd_out);
+
+ Backd(fd_out,n0);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* LE_GET_CURRENT_WORD                                                     */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+void LE_Get_Current_Word(char *word)
+
+{
+ char *str=global_str;
+ char *pos=global_pos;
+ char *end=global_end;
+ char *p,*q;
+
+ p=Skip(pos,str,0,-1);                              /* skip non separators */
+ if (Is_A_Separator(*p))
+     p++;
+
+ q=Skip(pos,end,0,+1);                              /* skip non separators */
+
+ while(p<q)
+     *word++ = *p++;
+
+ *word='\0';
+}
+
+
+
+
 /*-------------------------------------------------------------------------*/
 /* LE_GET_SEPARATORS                                                       */
 /*                                                                         */
@@ -1143,8 +1285,8 @@ static void Completion_Print_All(int fd_in,int fd_out)
  int       nb_in_a_line,nb_lines;
  int       nb_in_last_line,nb_miss_in_last_line;
  int       spaces,skip;
- int       k,s;
- char      buff[80];
+ int       k;
+ char      buff[512];
  int       l,c;
 
 
@@ -1187,15 +1329,15 @@ static void Completion_Print_All(int fd_in,int fd_out)
          if (++c==((l<nb_lines-1) ? nb_in_a_line : nb_in_last_line))
              break;
 
-         for(s=0;s<comp_match_max_lg-p1->word_length+spaces;s++)
-             LE_Put_Char(' ',fd_out);
+         sprintf(buff,"%*s",comp_match_max_lg-p1->word_length+spaces,"");
+         Display_String(fd_out,buff);
 
          skip=nb_lines;
          if (c>nb_in_a_line-nb_miss_in_last_line)
              skip--;
          for(k=0;k<skip;k++)
              p1=p1->next;
-	}
+        }
 
      NewLn();
      if (++l==nb_lines)
@@ -1215,6 +1357,14 @@ static void Completion_Print_All(int fd_in,int fd_out)
 static void Display_String(int fd_out,char *s)
 
 {
+#ifdef W32_GUI_CONSOLE
+ if (w32gc_display_string)
+    {
+     (*w32gc_display_string)(s);
+     return;
+    }
+#endif
+
  while(*s)
      LE_Put_Char(*s++,fd_out);
 }
@@ -1235,7 +1385,7 @@ static void Display_Help(int fd_out)
 
  L("")
  sprintf(buff,
-   "   linedit %-25s Copyright (C) 1999 Daniel Diaz",
+   "   linedit %-25s Copyright (C) 1999,2000 Daniel Diaz",
    LINEDIT_VERSION);
  L(buff)
  L("")
@@ -1294,10 +1444,38 @@ start:
      goto start;
     }
 
- if (c==KEY_EOF)
+ if (KEY_IS_EOF(c))
      c=EOF;
 
  LE_Term(fd_in,fd_out);
 
  return c;
 }
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* LE_PRINTF                                                               */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+int LE_Printf(char *format, ...)
+
+{
+ va_list arg_ptr;
+ int     ret;
+
+ va_start(arg_ptr,format);
+ if (w32gc_present)
+    {
+     char buff[1024];
+     ret=vsprintf(buff,format,arg_ptr);
+     (*w32gc_display_string)(buff);
+    }
+  else
+     ret=vprintf(format,arg_ptr);
+ va_end(arg_ptr);
+
+ return ret;
+}
+
