@@ -6,7 +6,7 @@
 /* Descr.: test file                                                       */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -22,12 +22,24 @@
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <signal.h>
+#if defined(__unix__) || defined(__CYGWIN__)
 #include <unistd.h>
+#else
+#include <io.h>
+#include <process.h>
+#endif
+
 
 #include "linedit.h"
 
+#define printf LE_Printf
+
+
+
+
 /*---------------------------------*/
 /* Constants                       */
 /*---------------------------------*/
@@ -36,7 +48,6 @@
 /* Type Definitions                */
 /*---------------------------------*/
 
-
 /*---------------------------------*/
 /* Global Variables                */
 /*---------------------------------*/
@@ -45,6 +56,9 @@
 /* Function Prototypes             */
 /*---------------------------------*/
 
+
+
+
 /*-------------------------------------------------------------------------*/
 /* INTERRUPT_HANDLER                                                       */
 /*                                                                         */
@@ -53,20 +67,24 @@ void Interrupt_Handler(int sig)
 
 {
  int       c;
- sigset_t  set;
  char      prefix[100];
  char     *str;
  int       nb,max_lg,is_last;
-
+#if defined(__unix__) || defined(__CYGWIN__)
+ sigset_t  set;
+#endif
 
  printf("\nCatching signal:%d\n",sig);
- signal(sig,Interrupt_Handler);
- printf("e to exit - c to continue - C for completions:");
+ printf("e to exit - c to continue - C for completions - w for current word:");
  fflush(stdout);
 
+#if defined(__unix__) || defined(__CYGWIN__)
  sigemptyset(&set);
  sigaddset(&set,sig);
  sigprocmask(SIG_UNBLOCK,&set,NULL);
+#else
+ signal(sig,Interrupt_Handler);
+#endif
 
  c=LE_FGetc_No_Echo(0,1);
  printf("\n");
@@ -74,6 +92,13 @@ void Interrupt_Handler(int sig)
  if (c=='e')
     exit(0);
 
+ if (c=='w')
+    {
+     LE_Get_Current_Word(prefix);
+     printf("current word=<%s>\n",prefix);
+     return;
+    }
+
  if (c=='C')
     {
      printf("Enter a prefix:");
@@ -82,11 +107,11 @@ void Interrupt_Handler(int sig)
      if ((str=LE_Compl_Init_Match(prefix,&nb,&max_lg))==NULL)
          printf("no matching\n");
       else
-	{
+        {
          printf("common=<%s> nb=%d max_lg=%d\n",str,nb,max_lg);
          while((str=LE_Compl_Find_Match(&is_last))!=NULL)
              printf("matching: <%s>\n",str);
-	}
+        }
     }
 }
 
@@ -97,16 +122,20 @@ void Interrupt_Handler(int sig)
 /* MAIN                                                                    */
 /*                                                                         */
 /*-------------------------------------------------------------------------*/
-int main()
+int main(int argc,char *argv[])
 
 {
- char line[256];
+ char  line[256];
  char *p;
- char sep[100];
+ char  sep[100];
+
+#if defined(_MSC_VER)
+ setbuf(stdout,NULL);
+ setbuf(stderr,NULL);
+#endif
 
 #if 1
  signal(SIGINT,Interrupt_Handler);
- signal(SIGQUIT,Interrupt_Handler);
 #endif
 
  sep[0]='\n';
@@ -125,18 +154,18 @@ int main()
      printf("enter a line:");
      if (LE_Gets(line)==NULL)
 #else
-     if (LE_FGets(line,50,0,1,"enter a line:",1)==NULL)
+     if (LE_FGets(line,200,0,1,"enter a line:",1)==NULL)
 #endif
          break;
      printf("Line:(%s) len:%d\n",line,strlen(line));
-     for(p=line;(p=strtok(p,sep));p=NULL)
+     for(p=line;(p=strtok(p,sep))!=NULL;p=NULL)
         {
          printf("adding word (%s) for completion\n",p);
          LE_Compl_Add_Word(strdup(p),strlen(p));
         }
     }
 
- return 0;
+ printf("End of testing\n");
+
+ return 12;
 }
-
-
