@@ -1,4 +1,4 @@
-#include "../../EnginePl/config.h"
+#include "../../EnginePl/gp_config.h"
 #include "../../EnginePl/pl_params.h"
 #include "../../EnginePl/obj_chain.h"
 #include "mach.h"
@@ -48,6 +48,7 @@ static void TRANS_code_start_non_global()
 void TRANS_pl_jump()
 {
  M_Direct_Goto(foo1);
+_foo1: ;
 }
 
 
@@ -56,9 +57,13 @@ void TRANS_pl_jump()
 
 void TRANS_pl_call()
 {
- CP =(long) pl_call1;
+ CP =(CodePtr) pl_call1;
  M_Direct_Goto(foo)
+#ifdef M_ix86_win32
+_foo: ;
+#else
  asm("pl_call1:");
+#endif
 }
 
 
@@ -95,7 +100,7 @@ void TRANS_jump()
      goto a;
     }
  x++;
-a:
+ a: ;
 }
 
 
@@ -168,7 +173,14 @@ void TRANS_call_c_foreign(void)
 void TRANS_jump_ret()
 
 {
+#if defined(M_ix86_win32)
+  register long adr=(long) bar (12,"toto");
+  _asm {
+    jmp adr
+  }
+#else
  goto *bar(12,"toto");
+#endif
 }
 
 
@@ -184,7 +196,7 @@ void TRANS_fail_ret()
      goto a;
 
  x++;
-a:
+a: ;
 }
 
 
@@ -256,7 +268,7 @@ void TRANS_cmp_ret_and_int()
      goto a;
 
  x++;
-a:
+a: ;
 }
 
 
@@ -269,7 +281,7 @@ void TRANS_jump_if_equal()
      goto a;
 
  x++;
-a:
+a: ;
 }
 
 
