@@ -2,11 +2,11 @@
 /* GNU Prolog                                                              */
 /*                                                                         */
 /* Part  : mini-assembler to assembler translator                          */
-/* File  : ix86_linux.c                                                    */
-/* Descr.: translation file for Linux on intel x86                         */
+/* File  : ix86_any.c                                                      */
+/* Descr.: translation file for Linux/Cygwin/... on intel x86              */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -23,7 +23,7 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <stdarg.h>
+#include <string.h>
 
 #include "../EnginePl/pl_params.h"
 #include "../EnginePl/obj_chain.h"
@@ -39,7 +39,7 @@
 
 #define MAX_C_ARGS_IN_C_CODE       32
 
-#ifdef M_ix86_cygwin
+#if defined(M_ix86_cygwin) || defined(M_ix86_bsd)
 
 #define UN                         "_"
 
@@ -179,7 +179,7 @@ void Code_Start(char *label,int prolog,int global)
 
 {
  Label_Printf("");
-#ifdef M_ix86_linux
+#if defined(M_ix86_linux) || defined(M_ix86_sco)
  Inst_Printf(".align","16");
  Inst_Printf(".type",UN "%s,@function",label);
 #else
@@ -703,7 +703,9 @@ void C_Ret(void)
 void Dico_String_Start(int nb_consts)
 
 {
+#if !defined(M_ix86_bsd)
  Label_Printf(".section\t.rodata");
+#endif
 }
 
 
@@ -761,7 +763,7 @@ void Dico_Long(char *name,int global,VType vtype,long value)
      case NONE:
          value=1;                               /* then in case ARRAY_SIZE */
      case ARRAY_SIZE:
-#if M_ix86_linux
+#if defined(M_ix86_linux) || defined(M_ix86_sco)
          if (!global)
              Inst_Printf(".local",UN "%s",name);
          Inst_Printf(".comm",UN "%s,%d,4",name,value*4);
