@@ -6,7 +6,7 @@
 /* Descr.: code generation                                                 */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -29,6 +29,7 @@
 #include "ma_parser.h"
 #include "ma_protos.h"
 
+#include "../EnginePl/gp_config.h"
 #include "../Wam2Ma/bt_string.c"
 #include "../TopComp/copying.c"
 
@@ -39,7 +40,7 @@
 /* Constants                       */
 /*---------------------------------*/
 
-#define DEFAULT_OUTPUT_SUFFIX      ".s"
+#define DEFAULT_OUTPUT_SUFFIX      ASM_SUFFIX
 
 #define LONG_STACK_SIZE            8192
 
@@ -214,7 +215,7 @@ void Call_C(char *fct_name,int nb_args,ArgInf arg[])
   else
      i=nb_args-1,inc=-1;
 
- for(;i<nb_args;i+=inc)
+ for(;i<(unsigned) nb_args;i+=inc)
     {
      switch(arg[i].type)
         {
@@ -255,6 +256,7 @@ void Call_C(char *fct_name,int nb_args,ArgInf arg[])
              break;
 
          default:                                      /* for the compiler */
+             ;
         }
     }
 
@@ -271,7 +273,8 @@ void Call_C(char *fct_name,int nb_args,ArgInf arg[])
 void Switch_Ret(int nb_swt,SwtInf swt[])
 
 {
- qsort((void *) swt,nb_swt,sizeof(SwtInf),(int(*)())Switch_Cmp_Int);
+ qsort((void *) swt,nb_swt,sizeof(SwtInf),
+       (int(*)(const void *,const void *)) Switch_Cmp_Int);
 
  Switch_Rec(0,nb_swt-1,swt);
 }
