@@ -6,7 +6,7 @@
 /* Descr.: mini-assembler parser                                           */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -698,7 +698,7 @@ void Syntax_Error(char *s)
  if (*p=='\n')
      *p='\0';
 
- fprintf(stderr,"line %d: \%s\n",cur_line_no,s);
+ fprintf(stderr,"line %d: %s\n",cur_line_no,s);
  fprintf(stderr,"%s\n",cur_line_str);
 
  for(p=cur_line_str;p<beg_last_token;p++)
