@@ -6,7 +6,7 @@
 /* Descr.: translation file for Linux on PowerPC                           */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -23,7 +23,7 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <stdarg.h>
+#include <string.h>
 
 #include "../EnginePl/pl_params.h"
 #include "../EnginePl/obj_chain.h"
