@@ -6,7 +6,7 @@
 /* Descr.: main file                                                       */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -371,9 +371,9 @@ cmd_line_arg1(PlFile,LArg,LArg):-
           /*--- copying ---*/
 
 display_copying:-
-	current_prolog_flag(prolog_name,Name),
-	current_prolog_flag(prolog_version,Version),
-	current_prolog_flag(prolog_copyright,Copyright),
+	prolog_name(Name),
+	prolog_version(Version),
+	prolog_copyright(Copyright),
 	format('Prolog to Wam Compiler (~a) ~a~n',[Name,Version]),
 	format('By Daniel Diaz~n',[]),
 	write(Copyright), nl,
@@ -385,6 +385,21 @@ display_copying:-
 
 
 
+prolog_name(Name):-
+	current_prolog_flag(prolog_name,Name).
+
+prolog_version(Version):-
+	current_prolog_flag(prolog_version,Version).
+
+prolog_date(Date):-
+	current_prolog_flag(prolog_date,Date).
+
+prolog_copyright(Copyright):-
+	current_prolog_flag(prolog_copyright,Copyright).
+
+
+
+
           /*--- help ---*/
 
 h('Usage: pl2wam [OPTION...] FILE').
