@@ -6,7 +6,7 @@
 /* Descr.: code emission                                                   */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -86,9 +86,9 @@ emit_code_init(WamFile0,PlFile0):-
                       ;  open(WamFile,write,Stream)),
 	g_assign(streamwamfile,Stream),
 	g_assign(cur_pl_file,''),
-	current_prolog_flag(prolog_name,Name),
-	current_prolog_flag(prolog_version,Version),
-	current_prolog_flag(prolog_date,Date),
+	prolog_name(Name),
+	prolog_version(Version),
+	prolog_date(Date),
 	date_time(dt(Yr,Mh,Dy,Hr,Me,Sd)),
 	format(Stream,'%% compiler: ~a ~a (~a)~n',[Name,Version,Date]),
 	format(Stream,'%% file    : ~a~n',[PlFile]),
