@@ -6,7 +6,7 @@
 /* Descr.: compiler main (shell) program                                   */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -21,29 +21,48 @@
 /* with this program; if not, write to the Free Software Foundation, Inc.  */
 /* 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     */
 /*-------------------------------------------------------------------------*/
+#include "../EnginePl/gp_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <dirent.h>
-#include <unistd.h>
 #include <errno.h>
 #include <ctype.h>
+#include <sys/types.h>
+
+#ifdef M_ix86_win32
+#include <windows.h>
+#include <process.h>
+#include <fcntl.h>
+#include <io.h>
+#define StrStr(s1,s2) Str_Case_Str(s1,s2)
+char *Str_Case_Str(char *s1,char *s2);
+#else
+#include <dirent.h>
+#include <unistd.h>
 #include <sys/param.h>
-#include <sys/types.h>
-
-#include <sys/types.h>
 #include <sys/wait.h>
-
-#include "decode_hexa.c"
-#include "copying.c"
-#include "../EnginePl/config.h"
+#define StrStr(s1,s2) strstr(s1,s2)
+#endif
 
 #ifdef M_ix86_cygwin
 #include <process.h>
 #endif
 
+#include "../EnginePl/arch_dep.h"
+#include "../EnginePl/wam_regs.h"
+
+#include "decode_hexa.c"
+#include "copying.c"
+
+#include "../EnginePl/machine1.c"
+
+#if 0
+#define DEBUG
+#endif
+
 
 
 
@@ -51,7 +70,7 @@
 /* Constants                       */
 /*---------------------------------*/
 
-#define MAX_SUB_DIRS               1024
+#define MAX_SUB_DIRS               128
 
 #define MAX_FILES                  1024
 
@@ -70,11 +89,11 @@
 #define EXE_FILE_PL2WAM            "pl2wam"
 #define EXE_FILE_WAM2MA            "wam2ma"
 #define EXE_FILE_MA2ASM            "ma2asm"
-#define EXE_FILE_ASM               "as"
+#define EXE_FILE_ASM               AS
 #define EXE_FILE_FD2C              "fd2c"
 #define EXE_FILE_CC                CC
 #define EXE_FILE_LINK              CC
-#define EXE_FILE_STRIP             "strip"
+#define EXE_FILE_STRIP             STRIP
 
 
 
@@ -97,17 +116,12 @@
 #define WAM_SUFFIX                 ".wam"
 #define WBC_SUFFIX                 ".wbc"
 #define MA_SUFFIX                  ".ma"
-#define ASM_SUFFIX                 ".s"
 #define FD_SUFFIX                  ".fd"
 #define C_SUFFIX                   ".c"
 #define C_SUFFIX_ALTERNATE         "|.C|.CC|.cc|.cxx|.c++|.cpp|"
-#define OBJ_SUFFIX                 ".o"
 
-#define CC_COMPILE_ONLY_OPT        "-c "
+#define CC_COMPILE_OPT             "-c "
 #define CC_INCLUDE_OPT             "-I"
-#define CC_OUTPUT_OPT              "-o "
-#define LINK_OUTPUT_OPT            "-o "
-
 
 
 
@@ -158,13 +172,14 @@ int      def_cstr_size  =-1;
 int      fixed_sizes=0;
 int      needs_stack_file=0;
 
-int      bc_mode     =0;
-int      no_top_level=0;
-int      min_pl_bips =0;
-int      min_fd_bips =0;
-int      no_debugger =0;
-int      no_fd_lib   =0;
-int      strip       =0;
+int      bc_mode       =0;
+int      gui_console   =0;
+int      no_top_level  =0;
+int      min_pl_bips   =0;
+int      min_fd_bips   =0;
+int      no_debugger   =0;
+int      no_fd_lib     =0;
+int      strip         =0;
 
 int      no_decode_hex=0;
 
@@ -178,8 +193,8 @@ CmdInf   cmd_wam2ma={ EXE_FILE_WAM2MA," ","-o " };
 CmdInf   cmd_ma2asm={ EXE_FILE_MA2ASM," ","-o " };
 CmdInf   cmd_asm   ={ EXE_FILE_ASM,   " ","-o " };
 CmdInf   cmd_fd2c  ={ EXE_FILE_FD2C,  " ","-o " };
-CmdInf   cmd_cc    ={ EXE_FILE_CC,    " ",CC_OUTPUT_OPT };
-CmdInf   cmd_link  ={ EXE_FILE_LINK,  " ",LINK_OUTPUT_OPT };
+CmdInf   cmd_cc    ={ EXE_FILE_CC,    " ",CC_OBJ_NAME_OPT };
+CmdInf   cmd_link  ={ EXE_FILE_LINK,  " ",CC_EXE_NAME_OPT };
 
 char    *cc_fd2c_flags=CFLAGS " ";
 
@@ -200,6 +215,7 @@ char    *suffixes[]=
 char     *Search_Path           (char *file);
 
 void      Init_Develop_Dir      (void);
+void      Init_Develop_Dir_Rec  (char *path, char ***p);
 
 void      Determine_Pathnames   (void);
 void      Compile_Files         (void);
@@ -208,6 +224,7 @@ void      Free_Work_File2       (FileInf *f);
 void      Compile_Cmd           (CmdInf *c,FileInf *f);
 void      Link_Cmd              (void);
 void      Exec_One_Cmd          (char *str,int no_decode_hex);
+int       Spawn_Decode_Hex      (char *arg[]);
 void      Delete_Temp_File      (char *name);
 
 void      Find_File             (char *file,char *suff,char *file_path);
@@ -248,21 +265,41 @@ int main(int argc,char *argv[])
 
 {
  static
- char  resolved[MAXPATHLEN];
+ char  resolved[MAXPATHLEN+1]; /* realpath() fills up to MAXPATHLEN (NULL) */
  static
  char  buff[MAXPATHLEN];
  char *p;
 
+#ifdef M_ix86_win32
+ setbuf(stdout,NULL);
+ setbuf(stderr,NULL);
+#endif
+
+ Parse_Arguments(argc,argv);
+ if (verbose)
+     fprintf(stderr,"\n");
+
  if ((start_path=getenv(ENV_VARIABLE))!=NULL)
      goto path_found;
 
  strcpy(buff,argv[0]);
- if (strcmp(buff+strlen(buff)-strlen(EXE_SUFFIX),EXE_SUFFIX)!=0)
+
+#ifdef DEBUG
+ fprintf(stderr,"value of argv[0]: <%s>\n",buff);
+#endif
+
+ if (strcasecmp(buff+strlen(buff)-strlen(EXE_SUFFIX),EXE_SUFFIX)!=0)
      strcat(buff,EXE_SUFFIX);
 
+#ifdef DEBUG
+ fprintf(stderr,"value of argv[0] with suffix: <%s>\n",buff);
+#endif
+
+#ifndef M_ix86_win32
  if (access(buff,X_OK)==0)
      start_path=buff;
   else
+#endif
      if ((start_path=Search_Path(buff))==NULL)
          goto path_not_found;
 
@@ -273,16 +310,26 @@ int main(int argc,char *argv[])
  strcpy(resolved+1,start_path);                       /* +1 for eventual @ */
 #endif
 
- sprintf(buff,"/bin/%s",GPLC);
- if ((p=strstr(resolved+1,buff))!=NULL)
+#ifdef DEBUG
+ fprintf(stderr,"path of the executable: %s\n",start_path);
+#endif
+
+ sprintf(buff,DIR_SEP_S "bin" DIR_SEP_S "%s",GPLC);
+
+ if ((p=StrStr(resolved+1,buff))!=NULL)
     {
      *p='\0';
      start_path=resolved+1;
      goto path_found;
     }         
 
- sprintf(buff,"/TopComp/%s",GPLC);
- if ((p=strstr(resolved+1,buff))!=NULL)
+ sprintf(buff,DIR_SEP_S "TopComp" DIR_SEP_S "%s",GPLC);
+
+#ifdef DEBUG
+ fprintf(stderr,"path of the executable: %s\n",start_path);
+ fprintf(stderr,"sub-path to search:     %s\n",buff);
+#endif
+ if ((p=StrStr(resolved+1,buff))!=NULL)
     {
      *p='\0';
      start_path=resolved;
@@ -301,17 +348,16 @@ path_found:
      devel_mode=1;
     }
 
- strcat(cmd_cc.opt,CFLAGS_MACHINE " " CFLAGS_REGS CC_COMPILE_ONLY_OPT);
+ strcat(cmd_cc.opt,CFLAGS_MACHINE " " CFLAGS_REGS CC_COMPILE_OPT);
 
  if (devel_mode)
      Init_Develop_Dir();
   else
-     sprintf(cmd_cc.opt+strlen(cmd_cc.opt),"%s%s/include ",
+     sprintf(cmd_cc.opt+strlen(cmd_cc.opt),"%s%s" DIR_SEP_S "include ",
              CC_INCLUDE_OPT,start_path);
 
  strcat(cmd_link.opt,LDFLAGS " ");
 
- Parse_Arguments(argc,argv);
  if (verbose)
      fprintf(stderr,"Path used: %s %s\n",start_path,
      (devel_mode) ? "(development mode)" : "");
@@ -331,7 +377,7 @@ path_found:
 char *Search_Path(char *file)
 
 {
-#if defined(__unix__) || defined(__CYGWIN__)
+#ifndef M_ix86_win32
 
  char *path=getenv("PATH");
  char *p;
@@ -356,7 +402,7 @@ char *Search_Path(char *file)
          l=strlen(buff);
         }
 
-     buff[l++]='/';
+     buff[l++]=DIR_SEP_C;
 
      strcpy(buff+l,file);
 
@@ -370,8 +416,17 @@ char *Search_Path(char *file)
     }
 
  return NULL;
+
 #else
- return search_path(file);
+
+ static
+ char  buff[MAXPATHLEN];
+ char *file_part;
+
+ SearchPath(NULL,file,".exe",MAXPATHLEN,buff,&file_part);
+
+ return  buff;
+
 #endif
 }
 
@@ -385,34 +440,86 @@ char *Search_Path(char *file)
 void Init_Develop_Dir(void)
 
 {
+ char **p;
+
+ p=devel_dir;
+
+ Init_Develop_Dir_Rec(start_path,&p);
+
+ *p=NULL;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*/
+/* INIT_DEVELOP_DIR                                                        */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+void Init_Develop_Dir_Rec(char *path, char ***p)
+
+{
+#ifdef M_ix86_win32
+ HANDLE          h;
+ WIN32_FIND_DATA d;
+#else
  DIR            *dir;
  struct dirent  *cur_entry;
- char          **p;
  struct stat     info;
- static
+#endif
+ char           *name;
  char            buff[MAXPATHLEN];
 
 
- dir=opendir(start_path);
- if (dir==NULL)
-     Fatal_Error("Cannot access to %s",start_path);
+#if 0
+ fprintf(stderr,"Searching sub-dir in: %s\n",path);
+#endif
 
- p=devel_dir;
+#ifdef M_ix86_win32
+ sprintf(buff,"%s\\*.*",path);
+ h=FindFirstFile(buff,&d);
+ if (h==INVALID_HANDLE_VALUE)
+#else
+ dir=opendir(path);
+ if (dir==NULL)
+#endif
+     Fatal_Error("Cannot access to %s",path);
+
+
+#ifdef M_ix86_win32
+ do
+    {
+     name=d.cFileName;
+#else
  while((cur_entry=readdir(dir))!=NULL)
     {
-     sprintf(buff,"%s/%s",start_path,cur_entry->d_name);
-     if (*cur_entry->d_name!='.' && 
+     name=cur_entry->d_name;
+#endif
+     sprintf(buff,"%s" DIR_SEP_S "%s",path,name);
+     if (*name!='.' && 
+#ifdef M_ix86_win32
+         (d.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)!=0)
+#else
          stat(buff,&info)==0 && S_ISDIR(info.st_mode))
+#endif
         {
-         *p++=strdup(cur_entry->d_name);
+#ifdef DEBUG
+	  fprintf(stderr,"sub-directory for include: %s (full: %s)\n",name,buff);
+#endif
+         **p=strdup(buff);
+         (*p)++;
          sprintf(cmd_cc.opt+strlen(cmd_cc.opt),"%s%s ",
                  CC_INCLUDE_OPT,buff);
+         Init_Develop_Dir_Rec(buff,p);
         }
+#ifdef M_ix86_win32
+    }
+ while(FindNextFile(h,&d)!=0);
+ FindClose(h);
+#else
     }
- 
- *p=NULL;
-
  closedir(dir);
+#endif
 }
 
 
@@ -733,9 +840,23 @@ void Link_Cmd(void)
 
  strcat(buff,LDLIBS " ");
 
+ if (gui_console)
+    {  /* modify Linedit/Makefile.in to follow this list of linker objects */
+     Find_File(LIB_W32GUICONS,"",buff+strlen(buff));
+     strcat(buff," ");
+     Find_File("w32gc_interf.obj","",buff+strlen(buff));
+     strcat(buff," ");
+     Find_File("GUICons.res","",buff+strlen(buff));
+     strcat(buff," /link /subsystem:windows ");
+    }
+#ifdef M_ix86_win32
+  else
+     strcat(buff,"/link /subsystem:console ");
+#endif
+
  Exec_One_Cmd(buff,no_decode_hex);
 
- if (strip && *EXE_FILE_STRIP!='\0')
+ if (strip && *EXE_FILE_STRIP!=':' && *EXE_FILE_STRIP!='\0')
     {
      sprintf(buff,"%s %s%s",EXE_FILE_STRIP,file_name_out,EXE_SUFFIX);
      Exec_One_Cmd(buff,1);
@@ -752,125 +873,95 @@ void Link_Cmd(void)
 void Exec_One_Cmd(char *cmd,int no_decode_hex)
 
 {
-#if defined(__unix__) || defined(__CYGWIN__)
- char *arg[CMD_LINE_MAX_OPT];
- char *p;
- int   i=0;
- int   error;
- int   pipe_out[2];
- FILE *fpipe;
+#if 1
+ int   status;
  static
- char  buff[CMD_LINE_LENGTH];
+ char *arg[2]={ NULL, (char *) 1 };
+
+ arg[0]=cmd;
 
  Before_Cmd(cmd)
- p=cmd;
- for(;;)
-    {
-     while(*p==' ')
-         p++;
 
-     if (*p=='\0')
-         break;
-
-     arg[i++]=p;
-
-     while(*p!=' ' && *p!='\0')
-         p++;
-
-     if (*p=='\0')
-         break;
-
-     *p++='\0';
-    }
-
- arg[i]=NULL;
-
-
- if (no_decode_hex==0)
-    {
-     if (pipe(pipe_out)<0)
-         Fatal_Error("error trying to execute %s",cmd);
-    }
-#ifdef M_ix86_cygwin
+ if (no_decode_hex==1)
+     status=M_Spawn(arg);
   else
-    {
-     error=spawnvp(_P_WAIT,arg[0],(const char * const *) arg);
-     if (error== -1)
-        {
-         fprintf(stderr,"cannot execute ");
-         perror(arg[0]);
-        }
-     goto finish;
-    }
-#endif
+     status=Spawn_Decode_Hex(arg);
 
- fflush(stdout);
- fflush(stderr);
- error=vfork();
- if (error== -1)
-     Fatal_Error("cannot create a new process for %s",arg[0]);
-
- if (error==0)                                            /* child process */
+ if (status== -1)
     {
-     if (no_decode_hex==0)
-        {
-         close(pipe_out[0]);
-         if (pipe_out[1]!=1)
-            {
-             dup2(pipe_out[1],1);
-             close(pipe_out[1]);
-            }
-         dup2(1,2);
-        }
-     execvp(arg[0],arg);                          /* only returns on error */
-     fprintf(stderr,"cannot execute ");
+     fprintf(stderr,"error trying to execute ");
      perror(arg[0]);
-     exit(1);
     }
 
- if (no_decode_hex==0)
-    {
-     close(pipe_out[1]);
-     fpipe=fdopen(pipe_out[0],"rt");
+ if (status== -2)
+     fprintf(stderr,"error trying to execute %s: unknown error",arg[0]);
 
-     if (fpipe==NULL)
-         Fatal_Error("error trying to execute %s",cmd);
-
-     for(;;)
-        {
-         fgets(buff,sizeof(buff)-1,fpipe);
-         if (feof(fpipe))
-             break;
-
-         fputs(Decode_Hexa(buff,"predicate(%s)",1,1),stderr);
-        }
-
-     if (fclose(fpipe))
-         Fatal_Error("error after executing %s",cmd);
-    }
- wait(&error);
-#ifdef M_ix86_cygwin
-finish:
-#endif
- After_Cmd(error)
+ After_Cmd(status)
 
 #else
 
- int error;
+ int status;
 
  Before_Cmd(cmd)
- error=system(cmd);
- error >>= 8;
- if (error== -1 || error==127)
+#ifdef DEBUG
+ fprintf(stderr,"executing system() for: %s\n",cmd);
+#endif
+ status=system(cmd);
+ status >>= 8;
+ if (status== -1 || status==127)
      Fatal_Error("error trying to execute %s",cmd);
 
- After_Cmd(error)
+ After_Cmd(status)
 #endif
 }
 
 
 
 
+/*-------------------------------------------------------------------------*/
+/* SPAWN_DECODE_HEX                                                        */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+int Spawn_Decode_Hex(char *arg[])
+
+{
+ int    pid,status;
+ FILE  *f_out;
+ static
+ char   buff[CMD_LINE_LENGTH];
+
+ pid=M_Spawn_Redirect(arg,0,NULL,&f_out,&f_out);
+ if (pid== -1 || pid== -2)
+     return pid;
+
+ for(;;)
+    {
+     fgets(buff,sizeof(buff),f_out);
+     if (feof(f_out))
+         break;
+
+#ifndef DEBUG
+     fputs(Decode_Hexa(buff,"predicate(%s)",1,1),stderr);
+#else
+     fprintf(stderr,"piped line:%s",Decode_Hexa(buff,"predicate(%s)",1,1));
+#endif
+    }
+
+ if (fclose(f_out))
+     return -1;
+
+ status=M_Get_Status(pid);
+
+#ifdef DEBUG
+ fprintf(stderr,"error status: %d\n",status);
+#endif
+
+ return status;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*/
 /* DELETE_TEMP_FILE                                                        */
 /*                                                                         */
@@ -905,14 +996,14 @@ void Find_File(char *file,char *suff,char *file_path)
  sprintf(name,"%s%s",file,suff);
  if (!devel_mode)
     {
-     sprintf(file_path,"%s/lib/%s",start_path,name);
+     sprintf(file_path,"%s" DIR_SEP_S "lib" DIR_SEP_S "%s",start_path,name);
      if (access(file_path,F_OK)==0)
          return;
     }
   else
      for(p=devel_dir;*p;p++)
         {
-         sprintf(file_path,"%s/%s/%s",start_path,*p,name);
+         sprintf(file_path,"%s" DIR_SEP_S "%s",*p,name);
          if (access(file_path,F_OK)==0)
              return;
         }
@@ -923,6 +1014,34 @@ void Find_File(char *file,char *suff,char *file_path)
 
 
 
+#ifdef M_ix86_win32
+
+/*-------------------------------------------------------------------------*/
+/* STR_CASE_STR                                                            */
+/*                                                                         */
+/*-------------------------------------------------------------------------*/
+char *Str_Case_Str(char *s1,char *s2)
+
+{
+ int   l1=strlen(s1);
+ int   l2=strlen(s2);
+ char *end=s1+l1-l2;
+
+ while(s1<=end)
+    {
+     if (strncasecmp(s1,s2,l2)==0)
+         return s1;
+     s1++;
+    }
+
+ return NULL;
+}
+
+#endif
+
+
+
+
 /*-------------------------------------------------------------------------*/
 /* FATAL_ERROR                                                             */
 /*                                                                         */
@@ -948,11 +1067,11 @@ void Fatal_Error(char *format,...)
  for(f=file;f->name;f++)
     {
      if (f->work_name1 && f->work_name1!=f->name && 
-         (file_name_out==NULL || strcmp(f->work_name1,file_name_out)!=0))
+         (file_name_out==NULL || strcasecmp(f->work_name1,file_name_out)!=0))
          Delete_Temp_File(f->work_name1);
 
      if (f->work_name2 && f->work_name2!=f->name && 
-         (file_name_out==NULL || strcmp(f->work_name2,file_name_out)!=0))
+         (file_name_out==NULL || strcasecmp(f->work_name2,file_name_out)!=0))
          Delete_Temp_File(f->work_name2);
     }
 
@@ -1206,6 +1325,17 @@ void Parse_Arguments(int argc,char *argv[])
              continue;
             }
 
+         if (Check_Arg(i,"--gui-console"))
+            {
+#ifdef W32_GUI_CONSOLE
+             Record_Link_Warn_Option(i);
+             gui_console=1;
+#else
+             fprintf(stderr,"Warning: Win32 GUI Console not available\n");
+#endif
+             continue;
+            }
+
          if (Check_Arg(i,"--no-debugger"))
             {
              Record_Link_Warn_Option(i);
@@ -1274,17 +1404,17 @@ void Parse_Arguments(int argc,char *argv[])
      if ((f->suffix=strrchr(argv[i],'.'))==NULL)
           f->suffix=argv[i]+strlen(argv[i]);
 
-     if (strcmp(PL_SUFFIX_ALTERNATE,f->suffix)==0)
+     if (strcasecmp(PL_SUFFIX_ALTERNATE,f->suffix)==0)
          f->type=FILE_PL;
       else
-         if ((q=strstr(C_SUFFIX_ALTERNATE,f->suffix)) &&
+         if ((q=StrStr(C_SUFFIX_ALTERNATE,f->suffix)) &&
              q[-1]=='|' && q[strlen(f->suffix)]=='|')
              f->type=FILE_C;
           else
             {
              f->type=FILE_LINK;
              for(p=suffixes;*p;p++)
-                 if (strcmp(*p,f->suffix)==0)
+                 if (strcasecmp(*p,f->suffix)==0)
                     {
                      f->type=p-suffixes;
                      break;
@@ -1307,7 +1437,7 @@ void Parse_Arguments(int argc,char *argv[])
  if (f==file)
     {
      if (verbose)
-         exit(0);               /* -verbose with no files same as -version */
+         exit(0);             /* --verbose with no files same as --version */
       else
          Fatal_Error("no input file specified");
     }
@@ -1386,6 +1516,7 @@ void Display_Help(void)
  L("  --trail-size N              set default trail  stack size to N Kb")
  L("  --cstr-size N               set default cstr   stack size to N Kb")
  L("  --fixed-sizes               do not consult environment variables at run-time")
+ L("  --gui-console               link the Win32 GUI console")
  L("  --no-top-level              do not link the top-level (force --no-debugger)")
  L("  --no-debugger               do not link the Prolog/WAM debugger")
  L("  --min-pl-bips               link only used Prolog built-in predicates")
