@@ -6,7 +6,7 @@
 /* Descr.: code generation                                                 */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -27,6 +27,8 @@
 #include <string.h>
 
 #include "wam_parser.h"
+#include "../EnginePl/gp_config.h"
+#include "../EnginePl/arch_dep.h"
 #include "../EnginePl/pl_params.h"
 #include "../BipsPl/pred_supp.h"
 
