@@ -6,7 +6,7 @@
 /* Descr.: parser                                                          */
 /* Author: Daniel Diaz                                                     */
 /*                                                                         */
-/* Copyright (C) 1999 Daniel Diaz                                          */
+/* Copyright (C) 1999,2000 Daniel Diaz                                     */
 /*                                                                         */
 /* GNU Prolog is free software; you can redistribute it and/or modify it   */
 /* under the terms of the GNU General Public License as published by the   */
@@ -116,17 +116,17 @@ InstInf inst[]={
     { "unify_local_value",   F_unify_local_value,   1,{ X_Y } },
     { "unify_atom",          F_unify_atom,          1,{ ATOM } },
     { "unify_integer",       F_unify_integer,       1,{ INTEGER } },
-    { "unify_nil",           F_unify_nil,           0,{ } },
-    { "unify_list",          F_unify_list,          0,{ } },
+    { "unify_nil",           F_unify_nil,           0,{ 0 } },
+    { "unify_list",          F_unify_list,          0,{ 0 } },
     { "unify_structure",     F_unify_structure,     1,{ F_N } },
 
     { "allocate",            F_allocate,            1,{ INTEGER } },
-    { "deallocate",          F_deallocate,          0,{ } },
+    { "deallocate",          F_deallocate,          0,{ 0 } },
 
     { "call",                F_call,                1,{ F_N, } },
     { "execute",             F_execute,             1,{ F_N, } },
-    { "proceed",             F_proceed,             0,{ } },
-    { "fail",                F_fail,                0,{ } },
+    { "proceed",             F_proceed,             0,{ 0 } },
+    { "fail",                F_fail,                0,{ 0 } },
 
     { "label",               F_label,               1,{ INTEGER } },
 
@@ -137,7 +137,7 @@ InstInf inst[]={
 
     { "try_me_else",         F_try_me_else,         1,{ INTEGER } },
     { "retry_me_else",       F_retry_me_else,       1,{ INTEGER } },
-    { "trust_me_else_fail",  F_trust_me_else_fail,  0,{ } },
+    { "trust_me_else_fail",  F_trust_me_else_fail,  0,{ 0 } },
 
     { "try",                 F_try,                 1,{ INTEGER } },
     { "retry",               F_retry,               1,{ INTEGER } },
@@ -154,7 +154,7 @@ InstInf inst[]={
 
     { "foreign_call_c",      F_foreign_call_c,      7,{ ATOM,ATOM,ATOM,INTEGER,INTEGER,LST_ATOM_INTEGER,LST_INTEGER } },
 
-    { NULL,                  NULL,                  0,{ } }
+    { NULL,                  NULL,                  0,{ 0 } }
     };
 
 
@@ -645,7 +645,8 @@ static int Scanner(int complex_atom)
 
          if ((*p++=*cur_line_p++)=='\\')
             {
-             if (strchr("abfnrtv",*cur_line_p))      /* \a \b \f \n \r \t \v */
+             if (*cur_line_p=='\\' ||                               /* \\ */
+                 strchr("abfnrtv",*cur_line_p))    /* \a \b \f \n \r \t \v */
                  *p++=*cur_line_p++;
               else
                 {
@@ -656,7 +657,7 @@ static int Scanner(int complex_atom)
                     }
                   else
                      i=8;
-                 i=strtol(cur_line_p,&p1,i);        /* stop on the closing \ */
+                 i=strtol(cur_line_p,&p1,i);      /* stop on the closing \ */
                  cur_line_p=p1+1;
                  sprintf(p,"%03o",i);
                  p+=3;
@@ -745,7 +746,7 @@ void Syntax_Error(char *s)
  if (*p=='\n')
      *p='\0';
 
- fprintf(stderr,"line %d: \%s\n",cur_line_no,s);
+ fprintf(stderr,"line %d: %s\n",cur_line_no,s);
  fprintf(stderr,"%s\n",cur_line_str);
 
  for(p=cur_line_str;p<beg_last_token;p++)
